from dataclasses import dataclass
from enum import Enum
from typing import List


@dataclass
class AssetMetadata:
    """
    Description:
        Asset metadata item
        :param dataType: (String) data type of the value
        :param key: (String) key
        :param source: (String) source
        :param value: (String) value of metadata
    """
    dataType: str = None
    key: str = None
    source: str = None
    value: str = None


class CreateAsset:

    def __init__(self,
                 name: str,
                 assemblyId: int,
                 uid: str,
                 assetTypeId: int,
                 sourceTypeId: int,
                 isCustom: bool,
                 currentSnapshot: str,
                 snapshots: List[str] = [],
                 description: str = None,
                 parentId: int = None,
                 metadata: List[AssetMetadata] = []
                 ):
        """
        Description:
            Create asset class.
        :param name: name of the asset
        :param assemblyId: data source id in torch catalog
        :param uid: asset uid
        :param assetTypeId: asset type id
        :param sourceTypeId: source type id
        :param isCustom: (bool) is custom asset
        :param currentSnapshot: current version of the datasource
        :param snapshots: version of the datasource
        :param description: description of the asset
        :param parentId: parent id of the asset
        :param metadata: (List[AssetMetadata]) metadata list of the asset
        """
        self.name = name
        self.description = description
        self.assemblyId = assemblyId
        self.uid = uid
        self.assetTypeId = assetTypeId
        self.sourceTypeId = sourceTypeId
        self.isCustom = isCustom
        self.currentSnapshot = currentSnapshot
        self.snapshots = snapshots
        if parentId is not None:
            self.parentId == parentId
        # metadata of an asset
        self.metadata = metadata

    def __eq__(self, other):
        return self.uid == other.uid

    def __repr__(self):
        return f"Asset({self.uid!r})"


class RelationType(Enum):
    """
        Description:
            Relation type between assets
    """
    FOREIGN_KEY = 1
    DEPENDS_ON = 2
    SIBLING = 3


class CreateAssetRelation:

    def __init__(self, fromAssetUUID: str, assemblyId: int, toAssetUUID: str, relationType: RelationType, metaData=None,
                 currentSnapshot=None, snapshots=[], **kwrgs):
        """
            Description:
                Used to create relation between any 2 assets
        :param fromAssetUUID: source asset uid
        :param assemblyId:  datasource id
        :param toAssetUUID: sink asset uid
        :param relationType: (RelationType) relation type b/w assets
        :param metaData: (List[AssetMetadata]) metadata list
        :param currentSnapshot: current version of the relation
        :param snapshots: versions of the datasource
        """
        self.fromAssetUUID = fromAssetUUID
        self.assemblyId = assemblyId
        self.toAssetUUID = toAssetUUID
        self.relationType = relationType
        self.metaData = metaData
        self.currentSnapshot = currentSnapshot
        self.snapshots = snapshots

    def __eq__(self, other):
        return self.toAssetUUID == other.toAssetUUID and self.fromAssetUUID == other.fromAssetUUID

    def __repr__(self):
        return f"CreateAssetRelation({self.__dict__})"



from torch_sdk.torch_client import TorchClient
from torch_sdk.models.tags import AssetLabel, CustomAssetMetadata
import torch_sdk.constants as const
from torch_sdk.models.profile import ProfilingType
from torch_sdk.models.profile import AutoProfileConfiguration, Profile, ProfileRequest, ProfilingType
import pprint
import time

from torch_sdk.models.pipeline import CreatePipeline, PipelineMetadata
import pprint
from datetime import datetime
import time
from torch_sdk.models.job import CreateJob, JobMetadata, Node

pp = pprint.PrettyPrinter(indent=4)
# pipeline_uid = "torch.airflow.demo.lambda"
# torch_credentials = {
#     'url': 'https://torchdemo.acceldata.tech/torch',
#     'access_key': 'V05DG18TV4MAF93',
#     'secret_key': 'VZF8O9Q5G6GUE08MQY5A8CA0TAE5KD'
# }
torch_credentials = {
    'url': 'https://torch.acceldata.local:5443/torch',
    'access_key': 'P04IM8FNQRUCRTU',
    'secret_key': 'E6LL9YUPMG4BDTJHT2VZD75HW0B8E5'
    # 'do_version_check' : True
}
# torch_credentials = {
#     'url': 'https://yubi.torch.acceldata.dev/torch',
#     'access_key': 'SAQ8XAOLSSLAD16',
#     'secret_key': 'KL5C4UA5XBQ2HBUMQPATJ3FB45HRH9'
# }
#### pipelines ###
torch_client = TorchClient(**torch_credentials)

pipeline_uid = "torch.airflow.test"
meta = PipelineMetadata(owner='sdk/pipeline-user', team='TORCH', codeLocation='...')
pipeline_name_ = "pipeline_name_test"
pipeline = CreatePipeline(
    uid=pipeline_uid,
    name=pipeline_name_,
    description=f'The pipeline {pipeline_name_} has been created from torch-sdk',
    meta=meta,
    context={'pipeline_uid': pipeline_uid, 'pipeline_name': pipeline_name_}
)
pipeline_res = torch_client.create_pipeline(pipeline=pipeline)
pipeline = torch_client.get_pipeline(pipeline_res.id)

pipeline_run = pipeline.create_pipeline_run()

job_uid = "test_job"
inputs = [Node(job_uid='customers.data-generation')]
outputs = [Node(asset_uid=f'S3-DS.s3_customers')]
context_job = {'job': 'data_gene', 'time': str(datetime.now()), 'uid': 'customers.data-generation',
               'operator': 'write_file_func'}
metadata = JobMetadata('Jason', 'COKE', 'https://github.com/coke/reports/customers.kt')
job = CreateJob(
    uid=job_uid,
    name=f'{job_uid} Job',
    version=pipeline_run.versionId,
    description=f'{job_uid} created using torch job decorator',
    inputs=inputs,
    outputs=outputs,
    meta=metadata,
    context=context_job
)
job = pipeline.create_job(job)
span = pipeline_run.create_span(uid=f'{pipeline_uid}.span')
#spans = pipeline_run.get_spans()
span_child = span.create_child_span(
    uid=f'{job_uid}.span',
    context_data={
        'time': str(datetime.now())
    },
    associatedJobUids=[job_uid])

exit()

pipeline = torch_client.get_pipeline(38)
pp.pprint('pipeline')
pp.pprint(pipeline)
runs = pipeline.get_runs()
pp.pprint('runs')
pp.pprint(runs)
pipeline_uid = torch_client.get_pipeline('shubh.airflow.coke.precreate13593')
pp.pprint('pipeline_uid')
pp.pprint(pipeline_uid)
pipelines = torch_client.get_pipelines()
pp.pprint('pipelines')
pp.pprint(pipelines)
pipelineRun = torch_client.get_pipeline_run(157)
pp.pprint('pipelineRun')
pp.pprint(pipelineRun)
pipeline_details = pipelineRun.get_details()
pp.pprint('pipeline_details')
pp.pprint(pipeline_details)
pipeline_spans = pipelineRun.get_spans()
pp.pprint('pipeline_spans')
pp.pprint(pipeline_spans)
pipeline = torch_client.get_pipeline(38)
delete_response = pipeline.delete()
#### pipelines ###
# import pdb;pdb.set_trace()
# async_executor = torch_client.execute_policy(const.PolicyType.DATA_QUALITY, 3, sync=True)
# xcom_key = f'{const.PolicyType.DATA_QUALITY.name}_{3}_execution_id'
# pp.pprint(xcom_key)
# if async_executor.errorMessage is not None:
#     async_execution_result = async_executor.get_result()

torch_credentials = {
    'url': 'https://test.torch1001.acceldata.dev/torch',
    'access_key': 'WKZW28UJGPHVJZJ',
    'secret_key': '5XP4HKAK0PBP00UQOUF89FEWDO7ZB6'
}

torch_client = TorchClient(**torch_credentials)
# asset = torch_client.get_asset('Automation_BIGQUERY_1.dq_bigquery_schema')
# pp.pprint('asset')
# pp.pprint(asset)
# ds_name = torch_client.get_datasource('gcs-111111', True)
# pp.pprint('datasource with name')
# pp.pprint(ds_name)
# ds = torch_client.get_datasource(265, True)
# pp.pprint('datasource with properties')
# pp.pprint(ds)
# dss = torch_client.get_datasources()
# pp.pprint('datasources')
# pp.pprint(dss)
# dss = torch_client.get_datasources(const.AssetSourceType.SNOWFLAKE)
# pp.pprint('datasources snowflake')
# pp.pprint(dss)
# asset = torch_client.get_asset(65240)
# pp.pprint('asset')
# pp.pprint(asset)
'''
response_start_crawler = ds.start_crawler()
pp.pprint('response_start_crawler')
pp.pprint(response_start_crawler)
response_status_crawler = ds.get_crawler_status()
pp.pprint('response_status_crawler')
pp.pprint(response_status_crawler)
'''

# metadata_asset = asset.get_metadata()
# pp.pprint('metadata_asset')
# pp.pprint(metadata_asset)
# sample_asset = asset.sample_data()
# pp.pprint('sample_asset')
# pp.pprint(sample_asset)
#
# asset.add_labels(labels=[AssetLabel('test12', 'shubh12'), AssetLabel('test22', 'shubh32')])
# labels_asset = asset.get_labels()
# pp.pprint('labels_asset')
# pp.pprint(labels_asset)
# asset.add_custom_metadata(custom_metadata=[CustomAssetMetadata('testcm1', 'shubhcm1'), CustomAssetMetadata('testcm2', 'shubhcm2')])
# latest_profile_status_asset = asset.get_latest_profile_status()
# pp.pprint('latest_profile_status_asset')
# pp.pprint(latest_profile_status_asset)
# start_profile_asset = asset.start_profile(ProfilingType.FULL)
# pp.pprint('start_profile_asset')
# pp.pprint(start_profile_asset)
# profile_status = start_profile_asset.get_status()
# pp.pprint('profile_status')
# pp.pprint(profile_status)
# time.sleep(5)
# cancel_res = profile_status = start_profile_asset.cancel()
# pp.pprint('cancel_res')
# pp.pprint(cancel_res)




# from torch_sdk.models.ruleExecutionResult import RuleType, PolicyFilter, ExecutionPeriod
# dq_policy_id = 1177
# # dq_rule_execution = torch_client.execute_policy(const.PolicyType.DATA_QUALITY, dq_policy_id)
# # pp.pprint('dq_rule_execution')
# # pp.pprint(dq_rule_execution)
#
# torch_client.get_policy(const.PolicyType.DATA_QUALITY, dq_policy_id)
# dq_rule_executions = torch_client.policy_executions(dq_policy_id, RuleType.DATA_QUALITY)
# pp.pprint('dq_rule_executions')
# pp.pprint(dq_rule_executions)
# dq_rule_executions = torch_client.policy_executions('null-check-bq-DQ-policy', RuleType.DATA_QUALITY)
# pp.pprint('dq_rule_executions')
# pp.pprint(dq_rule_executions)
# filter = PolicyFilter(policyType=RuleType.DATA_QUALITY, enable=True)
# dq_rules = torch_client.list_all_policies(filter=filter)
# pp.pprint(dq_rules)
# dq_rule = torch_client.get_policy(const.PolicyType.DATA_QUALITY, dq_policy_id)
# pp.pprint('dq_rule')
# pp.pprint(dq_rule)
# dq_rule_executions = dq_rule.get_executions()
# pp.pprint('dq_rule_executions')
# pp.pprint(dq_rule_executions)
# # async_executor = torch_client.execute_policy(const.PolicyType.DATA_QUALITY, 1114, sync=False)
# async_executor = dq_rule.execute(sync=False)
# if async_executor.errorMessage is None:
#     async_execution_status = async_executor.get_status()
#     pp.pprint('async_execution_status')
#     pp.pprint(async_execution_status)
#
#     async_execution_cancel = async_executor.cancel()
#     pp.pprint('async_execution_cancel')
#     pp.pprint(async_execution_cancel)
#     async_execution_result = async_executor.get_result()
#     pp.pprint('async_execution_result')
#     pp.pprint(async_execution_result)






class Notification:

    def __init__(self, associatedItemId, associatedItemType, createdAt, eventType, id, metadata: dict, title):
        self.title = title
        self.metadata = metadata
        self.id = id
        self.eventType = eventType
        self.createdAt = createdAt
        self.associatedItemType = associatedItemType
        self.associatedItemId = associatedItemId

    def __eq__(self, other):
        return self.id == other.id

    def __repr__(self):
        return f"Notification({self.__dict__})"


class Incident:

    def __init__(self, associatedItemId, associatedItemType, id, metadata: dict, raisedAt, resolver, sentTo, title, type):
        self.type = type
        self.title = title
        self.sentTo = sentTo
        self.resolver = resolver
        self.raisedAt = raisedAt
        self.metadata = metadata
        self.id = id
        self.associatedItemType = associatedItemType
        self.associatedItemId = associatedItemId

    def __eq__(self, other):
        return self.id == other.id

    def __repr__(self):
        return f"Incident({self.__dict__})"


import logging

from acceldata_sdk.events.job_events import JobStartEvent
from acceldata_sdk.events.log_events import LogEvent
from acceldata_sdk.initialiser import torch_client_credentials
from acceldata_sdk.models.pipeline import CreatePipeline, PipelineMetadata, PipelineRunResult, PipelineRunStatus
from acceldata_sdk.torch_client import TorchClient

logging.basicConfig(level=logging.INFO)

# Set up acceldata_sdk_code client
torch_client = TorchClient(url="https://torch.acceldata.local:5443",
                       access_key="N1LTYRK630PZ", secret_key="xPeUj4Iyj4WL2Tw284s9mqsgxvbPKW")

# create pipeline object
pipeline = CreatePipeline(
    uid='monthly_reporting-41',
    name='Monthly reporting Pipeline-41',
    description='Pipeline to create monthly reporting tables',
    meta=PipelineMetadata('vaishvik', 'acceldata_sdk_code', '...'),
    context={'key1': 'value1'}
)
# creating pipeline using acceldata_sdk_code client
pipeline_response = torch_client.create_pipeline(pipeline=pipeline)
print('Newly Created Pipeline Response : ', pipeline_response)


# create a pipeline run of the pipeline
pipelineRunResponse = pipeline_response.create_pipeline_run(context_data={'key1': 'value2', 'name': 'backend'})
print('pipeline Run Created :', pipelineRunResponse)

# update a pipeline run of the pipeline
update_pipeline_res = pipelineRunResponse.update_pipeline_run(context_data={'key1': 'value2', 'name': 'backend'},
                                                              result=PipelineRunResult.SUCCESS,
                                                              status=PipelineRunStatus.COMPLETED)
print('pipeline run updated :', update_pipeline_res)

# create span in the pipeline run
span_context = pipelineRunResponse.create_span(uid='span_uid_40')
print('Span Created :', span_context)

# start span event
# print('Span Start Event Response :', spanContext.start({'k': 'v'}))

# check current span is root or not
print('span is root or not : ', span_context.is_root())

job_start_event = JobStartEvent(context_data={'job':'start job'})
log_msg = """
21/08/26 10:02:06 ERROR PulseSinkReporter: Unable to report to Pulse
java.lang.RuntimeException: Fail to get Metric meta info
    at io.ad.pulse.agent.spark.sink.reporter.PulseSinkReporter.getMetricMeta(PulseSinkReporter.java:285)
    at io.ad.pulse.agent.spark.sink.reporter.PulseSinkReporter.reportCounter(PulseSinkReporter.java:228)
    at io.ad.pulse.agent.spark.sink.reporter.PulseSinkReporter.report(PulseSinkReporter.java:135)
    at com.codahale.metrics.ScheduledReporter.report(ScheduledReporter.java:162)
    at com.codahale.metrics.ScheduledReporter$1.run(ScheduledReporter.java:117)
    at java.util.concurrent.Executors$RunnableAdapter.call(Executors.java:511)
    at java.util.concurrent.FutureTask.runAndReset(FutureTask.java:308)
    at java.util.concurrent.ScheduledThreadPoolExecutor$ScheduledFutureTask.access$301(ScheduledThreadPoolExecutor.java:180)
    at java.util.concurrent.ScheduledThreadPoolExecutor$ScheduledFutureTask.run(ScheduledThreadPoolExecutor.java:294)
    at java.util.concurrent.ThreadPoolExecutor.runWorker(ThreadPoolExecutor.java:1149)
    at java.util.concurrent.ThreadPoolExecutor$Worker.run(ThreadPoolExecutor.java:624)
    at java.lang.Thread.run(Thread.java:748)
"""
log_event = LogEvent(log_data=log_msg)
# send log event for the current span
print('Span job start event Response :', span_context.send_event(span_event= job_start_event))
print('Span log event Response :', span_context.send_event(span_event= log_event))

# # send custom event
# print('Span Custom Event Response :', span_context.send_custom_event(event_type='CUSTOM_EVENT', context_data={'custom_key': 'custom_value'}))

# end the span event
print('Span End Event Response :', span_context.end())



import datetime
import time

from acceldata_sdk.models.job import CreateJob, JobMetadata, Dataset
from acceldata_sdk.models.pipeline import CreatePipeline, PipelineMetadata, PipelineRunResult, PipelineRunStatus
from acceldata_sdk.torch_client import TorchClient

def create_torch_client():
    # return TorchClient(
    #     url='https://acceldata.sso-demo.acceldata.dev/torch',
    #     access_key='1CXZ7RD61X1Z92F',
    #     secret_key='CKO7QJ2FZL7J12RIE06KUVRZSA2JOY'
    # )
    return TorchClient(
        url='https://a17bffb92ed5743d88e89cbfdd6a9c0c-1802836320.us-east-2.elb.amazonaws.com',
        access_key = '7GIF9LFARFX8',
        secret_key ='11ysBWOKU1t4eijydcldbd9kKiQcHV'
    )

def create_pipeline(torch_client):
    pipeline = CreatePipeline(
        # uid='pipeline.demo.first',
        # name='ETL PIPELINE - SDK ',
        uid='pipeline.demo.third',
        name='ETL PIPELINE - SDK - TORCH',
        description='Pipeline to Aggregate the customer orders over 1 year',
        meta=PipelineMetadata(
            owner='vaishvik', team='TORCH', codeLocation='www.v.co.in'),
        context={'tables': 'XYZ'}
    )
    pipeline_response = torch_client.create_pipeline(pipeline=pipeline)
    print('Created the pipeline')
    return pipeline_response


def create_datagen_job(pipeline):
    pipeline_run = pipeline.get_latest_pipeline_run()
    job = CreateJob(
        uid='datagen.job.1',
        name='DATAGEN JOB',
        version=pipeline_run.versionId,
        description='data gen job',
        inputs=[Dataset('AWS-S3-DS', 'airflow_order_details')],
        outputs=[
            Dataset('GCS-DS', 'customers')],
        meta=JobMetadata(owner='vaishvik', team='backend',
                         codeLocation='https://github.com/acme/reporting/reporting.scala'),
        context={}
    )
    job = pipeline.create_job(job)
    print('Created Job for random data insertion')
    return job


def create_insert_data_job(pipeline):
    pipeline_run = pipeline.get_latest_pipeline_run()
    job = CreateJob(
        uid='insert.data.job.2',
        name='INSERT DATA JOB',
        version=pipeline_run.versionId,
        description='insert data job',
        inputs=[Dataset('GCS-DS', 'customers')],
        outputs=[
            Dataset('AWS-RDS-DS', 'customers.public.products')],
        meta=JobMetadata(owner='vaishvik', team='backend',
                         codeLocation='https://github.com/acme/reporting/reporting.scala'),
        context={}
    )
    job = pipeline.create_job(job)
    print('Insert data Job for random data insertion')
    return job


def create_pipeline_run(pipeline):
    return pipeline.create_pipeline_run(
        context_data={'client_time': str(datetime.datetime.now())})


def start_main_span(pipeline_run):
    span_context = pipeline_run.create_span(
        uid='customer.orders.monthly.agg', context_data={'client_time': str(datetime.datetime.now())})
    return span_context


def end_main_span(span_context):
    span_context.end()


def end_pipeline_run(pipeline_run, result=PipelineRunResult.SUCCESS, status=PipelineRunStatus.COMPLETED):
    pipeline_run.update_pipeline_run(context_data={'client_time': str(datetime.datetime.now())},
                                     result=result,
                                     status=status)


if __name__ == "__main__":
    torch_client = create_torch_client()

    pipeline = create_pipeline(torch_client)
    pipeline_run = create_pipeline_run(pipeline)
    create_datagen_job(pipeline)
    create_insert_data_job(pipeline)
    span_context = start_main_span(pipeline_run)
    time.sleep(2)
    end_main_span(span_context)
    end_pipeline_run(pipeline_run)



from torch_sdk.torch_client import TorchClient
import test_constants as test_const
import pprint
import torch_sdk.constants as const
from torch_sdk.models.ruleExecutionResult import RuleType, PolicyFilter, ExecutionPeriod


pp = pprint.PrettyPrinter(indent=4)


class TestDQPolicy:
    torch_client = TorchClient(**test_const.torch_credentials)
    dq_policy_id = 1

    def test_get_policy(self):
        dq_rule = self.torch_client.get_policy(const.PolicyType.DATA_QUALITY, self.dq_policy_id)
        assert dq_rule is not None

    def test_get_all_policy(self):
        filter = PolicyFilter(policyType=RuleType.DATA_QUALITY, enable=True)
        dq_rules = self.torch_client.list_all_policies(filter=filter)
        assert dq_rules is not None

    def test_cancel_policy(self):
        dq_rule = self.torch_client.get_policy(const.PolicyType.DATA_QUALITY, self.dq_policy_id)
        async_executor = dq_rule.execute(sync=False)
        if async_executor.errorMessage is None:
            async_execution_status = async_executor.get_status()
            pp.pprint('async_execution_status')
            pp.pprint(async_execution_status)
            async_execution_cancel = async_executor.cancel()
            pp.pprint('async_execution_cancel')
            pp.pprint(async_execution_cancel)

    def test_execute_policy(self):
        dq_rule = self.torch_client.get_policy(const.PolicyType.DATA_QUALITY, self.dq_policy_id)
        async_executor = dq_rule.execute(sync=False)
        if async_executor.errorMessage is None:
            async_execution_status = async_executor.get_status()
            pp.pprint('async_execution_status')
            pp.pprint(async_execution_status)
            async_execution_result = async_executor.get_result()
            pp.pprint('async_execution_result')
            pp.pprint(async_execution_result)

    def test_get_policy_executions(self):
        dq_rule_executions = self.torch_client.policy_executions(self.dq_policy_id, RuleType.DATA_QUALITY)
        assert dq_rule_executions is not None


class TestReconPolicy:
    torch_client = TorchClient(**test_const.torch_credentials)
    recon_policy_id = 3

    def test_get_policy(self):
        recon_rule = self.torch_client.get_policy(const.PolicyType.RECONCILIATION, self.recon_policy_id)
        assert recon_rule is not None

    def test_get_all_policy(self):
        filter = PolicyFilter(policyType=RuleType.RECONCILIATION, enable=True)
        recon_rules = self.torch_client.list_all_policies(filter=filter)
        assert recon_rules is not None

    def test_cancel_policy(self):
        recon_rule = self.torch_client.get_policy(const.PolicyType.RECONCILIATION, self.recon_policy_id)
        async_executor = recon_rule.execute(sync=False)
        if async_executor.errorMessage is None:
            async_execution_status = async_executor.get_status()
            pp.pprint('async_execution_status')
            pp.pprint(async_execution_status)
            async_execution_cancel = async_executor.cancel()
            pp.pprint('async_execution_cancel')
            pp.pprint(async_execution_cancel)

    def test_execute_policy(self):
        recon_rule = self.torch_client.get_policy(const.PolicyType.RECONCILIATION, self.recon_policy_id)
        async_executor = recon_rule.execute(sync=False)
        if async_executor.errorMessage is None:
            async_execution_status = async_executor.get_status()
            pp.pprint('async_execution_status')
            pp.pprint(async_execution_status)
            async_execution_result = async_executor.get_result()
            pp.pprint('async_execution_result')
            pp.pprint(async_execution_result)

    def test_get_policy_executions(self):
        recon_rule_executions = self.torch_client.policy_executions(self.recon_policy_id, RuleType.RECONCILIATION)
        assert recon_rule_executions is not None



from dataclasses import dataclass
from enum import Enum


class LabelType(Enum):
    ASSET = 'ASSET'
    RULE = 'RULE'
    DQ_RULE_ITEM = 'DQ_RULE_ITEM'
    RECONCILIATION_COLUMN_MAPPING = 'RECONCILIATION_COLUMN_MAPPING'
    POLICY_TEMPLATE_MEASURE = 'POLICY_TEMPLATE_MEASURE'
    USER_DEFINED_FUNCTION = 'USER_DEFINED_FUNCTION'


class TAG:

    def __init__(self, name, autoGenerated=False, protectedData=False, id=None, createdAt=None, updatedAt=None):
        self.name = name
        self.autoGenerated = autoGenerated
        self.id = id
        self.createdAt = createdAt
        self.updatedAt = updatedAt
        self.protectedData = protectedData

    def __eq__(self, other):
        return self.name == other.name

    def __repr__(self):
        return f"TAG({self.__dict__})"


class AssetTag:

    def __init__(self, name, assetId, autoTagged=False, protectedData=False, id=None, tagId=None):
        self.name = name
        self.autoTagged = autoTagged
        self.protectedData = protectedData
        self.assetId = assetId
        self.tagId = tagId
        self.id = id

    def __eq__(self, other):
        return self.name == other.name

    def __repr__(self):
        return f"AssetTag({self.__dict__})"


class Watch:

    def __init__(self, assetId, userId, id, createdAt, updatedAt):
        self.assetId = assetId
        self.userId = userId
        self.id = id
        self.createdAt = createdAt
        self.updatedAt = updatedAt

    def __eq__(self, other):
        return self.id == other.id

    def __repr__(self):
        return f"WatchAsset({self.__dict__})"


class AssetActivity:

    def __init__(self, commentCount, downVoteCount, upVoteCount, downvoted=False, upvoted=False):
        self.commentCount = commentCount
        self.downVoteCount = downVoteCount
        self.upVoteCount = upVoteCount
        self.upvoted = upvoted
        self.downvoted = downvoted

    def __repr__(self):
        return f"AssetActivity({self.__dict__})"


class AssetComment:
    def __init__(self, comment, createdAt, updatedAt, userName):
        self.userName = userName
        self.updatedAt = updatedAt
        self.createdAt = createdAt
        self.comment = comment

    def __repr__(self):
        return f"AssetComment({self.__dict__})"


class AssetLabel:

    def __init__(self, key, value, labelType: LabelType = None):
        self.value = value
        self.key = key
        if labelType is not None:
            self.labelType = LabelType(labelType)
        else:
            self.labelType = labelType

    def __eq__(self, other):
        return self.key == other.key and self.value == other.value

    def __repr__(self):
        return f"AssetLabel({self.__dict__})"

@dataclass
class CustomAssetMetadata:
    """
    Description:
        Custom Asset metadata item
        :param key: (String) key
        :param value: (String) value of custom metadata
    """
    key: str
    value: str


class User:

    def __init__(self, id, name, createdAt, updatedAt):
        self.updatedAt = updatedAt
        self.createdAt = createdAt
        self.name = name
        self.id = id

    def __eq__(self, other):
        return self.id == other.id

    def __repr__(self):
        return f"TorchUser({self.__dict__})"


class APIKeys:

    def __init__(self, accessKey, id, secretKey, updatedAt=None, createdAt=None):
        self.accessKey = accessKey
        self.secretKey = secretKey
        self.updatedAt = updatedAt
        self.createdAt = createdAt
        self.id = id

    def __eq__(self, other):
        return self.accessKey == other.accessKey

    def __repr__(self):
        return f"API-KEYS({self.__dict__})"


class ClassificationConfig:

    def __init__(self, name, description, value, defaultValue, classificationType, classification, enabled=False,
                 protectedData=False, id=None, **kwargs):
        self.name = name
        self.description = description
        self.value = value
        self.defaultValue = defaultValue
        self.classificationType = classificationType
        self.classification = classification
        self.enabled = enabled
        self.protectedData = protectedData
        self.id = id

    def __eq__(self, other):
        return self.id == other.id

    def __repr__(self):
        return f"Classification-config({self.__dict__})"



import snowflake.connector


def snowflake_create_conn():
    snowflake_username = "winash"
    snowflake_password = "myD33ksh@k@n@lu"
    snowflake_account = "kf71436.us-east-1"
    snowflake_warehouse = "COMPUTE_WH"
    snowflake_database = "FINANCE"
    snowflake_schema = "FINANCE"
    conn = snowflake.connector.connect(
        user=snowflake_username,
        password=snowflake_password,
        account=snowflake_account,
        warehouse=snowflake_warehouse,
        database=snowflake_database,
        schema=snowflake_schema
    )
    return conn


sf_table_namein = "FINANCE.BLACKROCK_QUARANTINE_RAW"
print("executing cleanup")
conn = snowflake_create_conn()
cur = conn.cursor()
print("executing cleanup on SF")

command = "update IDENTIFIER(%(tablename)s) set CARD_NUMBER = case when CARD_NUMBER is null then '5599-4639-3545-6532' else CARD_NUMBER  end, ACC_ID = case when ACC_ID is null then '80154107168255874003' else ACC_ID end, TYPE = case when TYPE='SHARATH' then 'RUPAY' else TYPE end"
bind_params = {
    "tablename": sf_table_namein,
}
results = cur.execute(command, bind_params)
conn.commit()
cur.close()
exit()


sf_table_namein = "finance.blackrock_raw_mirror"
conn = snowflake_create_conn()

cur = conn.cursor()
command = (
    "ALTER TABLE IDENTIFIER(%(tablename)s) DROP COLUMN \"ad_rule__143__result\"")

bind_params = {
    "tablename": sf_table_namein
}
results = cur.execute(command, bind_params)
conn.commit()
cur.close()


cur = conn.cursor()
print("executing truncate on SF")
command = ("TRUNCATE TABLE IDENTIFIER(%(tablename)s)")
bind_params = {
    "tablename": sf_table_namein,
}
results = cur.execute(command, bind_params)
row = cur.fetchall()
conn.commit()
cur.close()

cur = conn.cursor()
# get the rule_name and execution id - then pull them in xcom
stage_path_suffix = '2022-09-29/segmented_cc/_1664445283208/successrecords'
stage_name = '@shubh_ext_stage2/'
stage_load_data_path = stage_name + stage_path_suffix

# command = (
#     "copy into IDENTIFIER(%(tablename)s) from IDENTIFIER(%(stage_load_data_path)s)  FILE_FORMAT=(%(file_format)s) ON_ERROR=(%(on_error)s) MATCH_BY_COLUMN_NAME=(%(case_insensitive)s) PURGE=(%(purge)s)")
command = (
    "copy into IDENTIFIER(%(tablename)s) from %(stage_load_data_path)s  FILE_FORMAT='my_parquet_format' ON_ERROR='CONTINUE' MATCH_BY_COLUMN_NAME='CASE_INSENSITIVE' PURGE=FALSE")
# bind_params = {
#     "tablename": sf_table_namein,
#     "stage_load_data_path": stage_load_data_path,
#     "file_format": 'my_parquet_format',
#     "on_error": 'CONTINUE',
#     "case_insensitive": 'CASE_INSENSITIVE',
#     "purge": 'FALSE'
# }
bind_params = {
    "tablename": sf_table_namein,
    "stage_load_data_path": stage_load_data_path
}
results = cur.execute(command, bind_params)
conn.commit()
cur.close()



import logging

from acceldata_sdk.models.job import CreateJob, JobMetadata, Dataset
from acceldata_sdk.models.pipeline import CreatePipeline, PipelineMetadata, PipelineRunResult, PipelineRunStatus
from acceldata_sdk.torch_client import TorchClient

logging.basicConfig(level=logging.INFO)

# Set up acceldata_sdk_code client
torchClient = TorchClient(url="https://torch.acceldata.local:5443/torch",
                          access_key="SY1Q793UECO38GB", secret_key="JQ9CUKVZKD8JBDNHMLFMDW5E3VCF8U")

# create pipeline object
pipeline = CreatePipeline(
    uid='TEST_PIPELINE',
    name='test pipeline',
    description='Pipeline for testing',
    meta=PipelineMetadata('Vaishvik', 'acceldata_sdk_code', '...'),
    context={'Key': 'value'}
)
# creating pipeline using acceldata_sdk_code client
pipelineResponse = torchClient.create_pipeline(pipeline=pipeline)
print('Newly Created Pipeline Response : ', pipelineResponse)

# create job object
job = CreateJob(
    uid='JOB_1',
    name='MIGRATION JOB',
    pipelineSnapshot=pipeline.currentSnapshot,
    description='migration job',
    inputs=[Dataset('pg_ds', 'ad_catalog.ad_catalog.flyway_schema_history')],
    outputs=[Dataset('pg_ds_new', 'ad_catalog.ad_catalog.flyway_schema_history')],
    meta=JobMetadata('vaishvik', 'backend', 'https://github.com/acme/reporting/reporting.scala'),
    context={'key21': 'value21'}
)
# create a job on newly created pipeline
jobResponse = pipelineResponse.create_job(job)
print('Newly Created Job Response : ', jobResponse)

# create a pipeline run of the pipeline
pipelineRunResponse = pipelineResponse.create_pipeline_run()
print('pipeline Run Created :', pipelineRunResponse)

# create span in the pipeline run
spanContext = pipelineRunResponse.create_span(uid='span_uid_38')
print('Span Created :', spanContext)

# check current span is root or not
print('span is root or not : ', spanContext.is_root())

# get span
sc = pipelineRunResponse.get_span(span_uid='span_uid_38')
print('span con get : ', sc)

# end the span event
print('Span End Event Response :', spanContext.end())

# check if the current span has children or not
print('has child ', spanContext.has_children())

# # create a child span
childSpanContext = spanContext.create_child_span('span_uid_38_2')
print('child span abort event : ', childSpanContext.abort())

childSpans = spanContext.get_child_spans()
print('child spans list: ', childSpans, ' ; size : ', len(childSpans))
print('has child ', spanContext.has_children())

# update a pipeline run of the pipeline
updatePipelineRunRes = pipelineRunResponse.update_pipeline_run(context_data={'key1': 'value2', 'name': 'backend'},
                                                               result=PipelineRunResult.SUCCESS,
                                                               status=PipelineRunStatus.COMPLETED)
print('pipeline run updated :', updatePipelineRunRes)



from enum import Enum


class Profile:

    def __init__(self, assetId, createdAt, id, jobId, profilingType, sparkClusterContext, status, updatedAt, rows=None,
                 totalRows=None, jobType=None, autoProfileId=None, profileDataAvailable=None, marker=None,
                 profiledData=None, error=None, client=None, **kwargs):
        self.client = client
        self.autoProfileId = autoProfileId
        self.profileDataAvailable = profileDataAvailable
        self.jobType = jobType
        self.updatedAt = updatedAt
        self.status = status
        self.sparkClusterContext = sparkClusterContext
        self.rows = rows
        self.profilingType = profilingType
        self.profiledData = profiledData
        self.marker = marker
        self.jobId = jobId
        self.id = id
        self.error = error
        self.createdAt = createdAt
        self.assetId = assetId
        self.totalRows = totalRows

    def __repr__(self):
        return f"Profile({self.__dict__})"

    def cancel(self):
        return self.client.cancel_profile(self.id)

    def get_status(self):
        return self.client.get_profile_request_details(asset_id=self.assetId, req_id=self.id)


class AutoProfileConfiguration:

    def __init__(self, assemblyId, assemblyName, createdAt, updatedAt, enabled, schedule, parallelizationCount, id=None,
                 includeList=None, excludeList=None, **kwargs):
        self.assemblyId = assemblyId
        self.assemblyName = assemblyName
        self.createdAt = createdAt
        self.updatedAt = updatedAt
        self.enabled = enabled
        self.schedule = schedule
        self.parallelizationCount = parallelizationCount
        self.id = id
        if isinstance(includeList, list):
            self.includeList = list(includeList)
        else:
            self.includeList = includeList

        if isinstance(excludeList, dict):
            self.excludeList = list(excludeList)
        else:
            self.excludeList = excludeList

    def __eq__(self, other):
        return self.id == other.id

    def __repr__(self):
        return f"AutoProfileConfiguration({self.__dict__})"


class ProfilingType(Enum):
    SAMPLE = 'SAMPLE'
    FULL = 'FULL'
    INCREMENTAL = 'INCREMENTAL'


class JobType(Enum):
    PROFILE = 'profile'
    MINI_PROFILE = 'mini-profile'
    # FULL = 'autotag'


class ProfileRequest:

    def __init__(self, id, status, createdAt, updatedAt, client=None, **kwargs):
        self.client = client
        self.updatedAt = updatedAt
        self.createdAt = createdAt
        self.status = status
        self.id = id

    def __repr__(self):
        return f"ProfileRequest({self.__dict__})"

    def cancel(self):
        return self.client.cancel_profile(self.id)



from acceldata_sdk.initialiser import torch_client_credentials
from acceldata_sdk.models.asset import RelationType
from acceldata_sdk.models.datasource import CreateDataSource, DatasourceType
from acceldata_sdk.torch_client import TorchClient
from acceldata_sdk.models.create_asset import AssetMetadata

torchClient = TorchClient(url="https://torch.acceldata.local:5443",
                       access_key="N1LTYRK630PZ", secret_key="xPeUj4Iyj4WL2Tw284s9mqsgxvbPKW")

datasource = CreateDataSource(
    name='Feature_bag_datasource_sdk_3',
    sourceType=DatasourceType(21, 'FEATURE_BAG'),
    description='feature bag assembly creation using python sdk',
    isVirtual=True,
    # name= 'aws_athena_ds_sdk',
    # sourceType= DatasourceType(16, 'AWS_ATHENA'),
    # description= 'creation of assembly using sdk',
    # connectionId= 1,
    # configProperties= [ ConfigProperty(key= 's3.location', value= 's3://ad-redshift-data/athena_result/s3')]
)

datasourceResponse = torchClient.create_datasource(datasource)
print('Newly created datasource: ', datasource)

newSnapshotVersion = datasourceResponse.initialise_snapshot(uid='NIffc38-9daa-4842-b008-f7fb3dd8439a')
print('New snapshot version created : ', newSnapshotVersion)

# getCurrentVersion = datasourceResponse.get_current_snapshot()
# print('getting current version of datasource : ', getCurrentVersion)

metadata = [AssetMetadata('STRING', 'abcd', 'pqr', 'sds'), AssetMetadata('STRING', 'abcdq', 'pqrq', 'sqds'), AssetMetadata('STRING', 'abcddq', 'pqrqd', 'sqdds')]

assetResponse = datasourceResponse.create_asset(uid='Feature_bag_datasource_sdk.feature_sdk_1',
                                                metadata=metadata,
                                                asset_type_id=22,
                                                description='Test asset creation',
                                                name='feature_sdk_1'
                                                )
print('Newly created asset response : ', assetResponse)

# fromAssetUUID = 'Feature_bag_datasource_sdk.feature_sdk_1'
# toAssetUUID = 'postgres-assembly-5450.ad_catalog.ad_catalog.qrtz_simple_triggers'
# relationType = RelationType.SIBLING
#
# assetRelationResponse = assetResponse.create_asset_relation(relation_type= relationType, to_asset_uuid= toAssetUUID)
# print('Newly created asset relation', assetRelationResponse)

delete_asset = datasourceResponse.delete_asset(id=9523)
print(delete_asset)



from faker import Faker, Factory
from random import randint, choices
import string
import datetime
import snowflake.connector


def create_accounts():
    accounts = []
    fake = Faker()
    acc_type = ['savings', 'current', 'salary']
    linked = [True, False]

    for i in range(1, 10):
        account = {
            'acc_num': ''.join(["{}".format(randint(0, 9)) for num in range(0, 20)]),
            'customer_name': fake.name(),
            'unique_identity': ''.join(choices(string.ascii_uppercase, k=3)) + ''.join(choices(string.digits, k=4)),
            'phone_number': fake.phone_number(),
            'balance': randint(10000, 10000000),
            'acc_type': acc_type[randint(0, 2)],
            'upper_limit': randint(10000000, 20000000),
            'credit_cards_linked': linked[randint(0, 1)],
            'loan_amount': randint(100000, 1000000),
            'email': fake.email(),
            'last_login_at': datetime.datetime(2021, 5, 1) + datetime.timedelta(days=randint(25, 29)),
            'created_at': datetime.datetime(2021, 5, 1) + datetime.timedelta(days=randint(1, 24))
        }
        accounts.append(account)
    return accounts


def create_branches():
    branches = []
    fake = Faker()
    bank_type = ['PRIVATE', 'CENTRAL']

    for i in range(1, 10):
        branch = {
            'id': ''.join(choices(string.ascii_uppercase, k=2)) + ''.join(choices(string.digits, k=4)),
            'address': fake.address(),
            'ifsc_code': ''.join(choices(string.ascii_uppercase, k=2)) + ''.join(choices(string.digits, k=4)),
            'employees_count': randint(25, 100),
            'micr_code': ''.join(choices(string.ascii_uppercase, k=2)) + ''.join(choices(string.digits, k=4)),
            'established_on': datetime.datetime(randint(1995, 2021), randint(1, 11), randint(1, 30)),
            'type': bank_type[randint(0, 1)]
        }
        branches.append(branch)
    return branches


def create_credit_cards(account_numbers):
    cards = []
    card_type = ['VISA', 'MASTER', 'EXPRESS', 'RUPAY']

    if len(account_numbers) <= 0:
        return

    for acc_num in account_numbers:
        num = randint(1, 10)
        if num >= 3:
            card = {
                'card_number': str(randint(1000, 9999)) + '-' + str(randint(1000, 9999)) + '-' + str(
                    randint(1000, 9999)) + '-' + str(randint(1000, 9999)),
                'cvv': randint(100, 999),
                'valid_upto': str(randint(1, 12)) + '/' + str(randint(2020, 2028)),
                'acc_id': acc_num,
                'limit': randint(10000, 500000),
                'type': card_type[randint(0, 3)]
            }
            cards.append(card)

    return cards


def create_transactions(account_numbers):
    transactions = []
    debit = [True, False]
    fake = Faker()

    if len(account_numbers) <= 0:
        return

    for acc_num in account_numbers:
        num = randint(1, 10)
        if num >= 3:
            transaction = {
                'id': ''.join(choices(string.ascii_uppercase, k=3)) + ''.join(choices(string.digits, k=5)),
                'from_acc_id': acc_num,
                'to_acc_id': account_numbers[randint(0, len(account_numbers) - 1)],
                'amount': randint(100, 100000),
                'debit': debit[randint(0, 1)],
                'started_at': datetime.datetime(2021, 6, 1) + datetime.timedelta(days=randint(1, 20)),
                'merchant_name': fake.company(),
                'end_balance': randint(10000, 1000000)
            }
            transactions.append(transaction)
    return transactions


def create_conn():
    conn = snowflake.connector.connect(
        user='winash',
        password='myD33ksh@k@n@lu',
        account='kf71436.us-east-1',
        warehouse='COMPUTE_WH',
        database='CUSTOMERS_DATABASE',
        schema='CUSTOMERS'
    )
    return conn


def generate_account_data():
    insert_acc_sql = """
    INSERT INTO FINANCE.FINANCE."ACCOUNTS" (ACC_NUM, CUSTOMER_NAME, UNIQUE_IDENTITY, PHONE_NUMBER, BALANCE, ACC_TYPE, UPPER_LIMIT, CREDIT_CARDS_LINKED, LOAN_AMOUNT, EMAIL, CREATED_AT, LAST_LOGIN_AT) VALUES(%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s);
    """
    account_numbers = []
    accounts = create_accounts()
    conn = create_conn()
    cur = conn.cursor()

    for acc in accounts:
        cur.execute(insert_acc_sql, (
        acc['acc_num'], acc['customer_name'], acc['unique_identity'], acc['phone_number'], acc['balance'],
        acc['acc_type'], acc['upper_limit'], acc['credit_cards_linked'], acc['loan_amount'], acc['email'],
        acc['created_at'], acc['last_login_at'],))
        acc_num = acc['acc_num']
        account_numbers.append(acc_num)

    conn.commit()
    return account_numbers


def generate_branch_data():
    insert_branch_sql = """
    INSERT INTO FINANCE.FINANCE.BRANCHES (ID, ADDRESS, IFSC_CODE, EMPLOYEE_COUNT, MICR_CODE, ESTABLISHED_ON, "TYPE") VALUES(%s, %s, %s, %s, %s, %s, %s);
    """

    branches = create_branches()
    conn = create_conn()
    cur = conn.cursor()

    for branch in branches:
        cur.execute(insert_branch_sql, (
        branch['id'], branch['address'], branch['ifsc_code'], branch['employees_count'], branch['micr_code'],
        branch['established_on'], branch['type'],))

    conn.commit()


def generate_credit_card_data(account_numbers):
    insert_cc_sql = """
    INSERT INTO FINANCE.FINANCE.CREDIT_CARDS (CARD_NUMBER, CVV, VALID_UPTO, ACC_ID, "LIMIT", "TYPE") VALUES(%s, %s, %s, %s, %s, %s);
    """

    cards = create_credit_cards(account_numbers)
    conn = create_conn()
    cur = conn.cursor()

    for cc in cards:
        cur.execute(insert_cc_sql,
                    (cc['card_number'], cc['cvv'], cc['valid_upto'], cc['acc_id'], cc['limit'], cc['type'],))

    conn.commit()


def generate_transactions(account_numbers):
    insert_trans_sql = """
    INSERT INTO FINANCE.FINANCE.TRANSACTIONS_LOGS (ID, FROM_ACC_ID, TO_ACC_ID, AMOUNT, DEBIT, STARTED_AT, MERCHANT_NAME, END_BALANCE) VALUES(%s, %s, %s, %s, %s, %s, %s, %s);
    """

    trans = create_transactions(account_numbers)
    conn = create_conn()
    cur = conn.cursor()

    for tran in trans:
        cur.execute(insert_trans_sql, (
        tran['id'], tran['from_acc_id'], tran['to_acc_id'], tran['amount'], tran['debit'], tran['started_at'],
        tran['merchant_name'], tran['end_balance'],))

    conn.commit()


def generate_finance_data():
    print('Generating data')
    print('Accounts......')
    account_numbers = generate_account_data()
    print('Branches......')
    generate_branch_data()
    print('Credit cards......')
    generate_credit_card_data(account_numbers)
    print('transactions......')
    generate_transactions(account_numbers)


generate_finance_data()




from torch_client import TorchClient

torch_client = TorchClient(url="https://torch.acceldata.local:5443/torch",
                           access_key="P04IM8FNQRUCRTU", secret_key="E6LL9YUPMG4BDTJHT2VZD75HW0B8E5")

datasource_id = 2
import pdb;pdb.set_trace()
datasource_details = torch_client.get_datasource_by_id(datasource_id)
print(datasource_details)



from acceldata_sdk.errors import TorchSdkException
from acceldata_sdk.models.pipeline import CreatePipeline, Pipeline, PipelineRun
from acceldata_sdk.torch_http_client import TorchHttpClient
from acceldata_sdk.models.datasource import CreateDataSource
from acceldata_sdk.constants import RuleExecutionStatus, FailureStrategy, PolicyType, AssetSourceType, \
    SDK_MIN_VERSION_SUPPORT, SdkSupportedVersions
from acceldata_sdk.models.rule import RuleResource
from typing import List
from acceldata_sdk.models.ruleExecutionResult import RuleResult, PolicyFilter, RuleType
from semantic_version import Version, SimpleSpec
import logging
import distutils


class TorchClient:
    logger = logging.getLogger('torch')
    logger.setLevel(logging.INFO)

    """
            Description : Torch user client is used to send data to catalog server.

            :param url: (String) url of the catalog server
            :param timeout_ms: (Integer) timeout of the requests sending to catalog
            :param access_key: (String) Access key of API key. You can generate API key from torch UI's setting
            :param secret_key: (String) Secret key of API key.
            :param do_version_check: (bool) Enable version compatibility check between sdk and torch.


            Ex.  TorchClient = TorchUserClient(url='https://torch.acceldata.local:5443', access_key='OY2VVIN2N6LJ', secret_key='da6bDBimQfXSMsyyhlPVJJfk7Zc2gs')
    """

    def __init__(self, url, timeout_ms=10000, access_key: str = None, secret_key: str = None,
                 do_version_check: bool = True):
        """
                Description : Torch user client is used to send data to catalog server.

                :param url: (String) url of the catalog server
                :param timeout_ms: (Integer) timeout of the requests sending to catalog
                :param access_key: (String) Access key of API key. You can generate API key from torch UI's setting
                :param secret_key: (String) Secret key of API key.
                :param do_version_check: (bool) Enable version compatibility check between sdk and torch.


                Ex.  TorchClient = TorchUserClient(url='https://torch.acceldata.local:5443', access_key='OY2VVIN2N6LJ', secret_key='da6bDBimQfXSMsyyhlPVJJfk7Zc2gs')
        """
        if access_key is None and secret_key is None:
            raise Exception('Access key and secret key - required')
        self.client = TorchHttpClient(url=url, access_key=access_key, secret_key=secret_key, timeout_ms=timeout_ms)
        if isinstance(do_version_check, str):
            self.do_version_check = bool(distutils.util.strtobool(do_version_check))
        else:
            self.do_version_check = do_version_check
        if self.do_version_check:
            supported_versions = self.get_supported_sdk_versions()
            ver_comparator = SimpleSpec(f'>={supported_versions.minVersion},<={supported_versions.maxVersion}')
            if Version(SDK_MIN_VERSION_SUPPORT) not in ver_comparator:
                raise Exception(f'Torch supports sdk versions between {supported_versions.minVersion} and '
                                f'{supported_versions.maxVersion}')
        else:
            self.logger.info('Skipping version check')

    def get_supported_sdk_versions(self) -> SdkSupportedVersions:
        """
        Description:
            To get supported sdk versions
        :return: (SdkSupportedVersions) newly created SdkSupportedVersions class instance
        """
        return self.client.get_supported_sdk_versions()

    def create_pipeline(self, pipeline: CreatePipeline) -> Pipeline:
        """
        Description:
            To create pipeline in torch catalog service
        :param pipeline: (CreatePipeline) class instance of the pipeline to be created
        :return: (Pipeline) newly created pipeline class instance
        """
        if pipeline.uid is None or pipeline.name is None:
            raise Exception('To create a pipeline, pipeline uid/name is required')
        return self.client.create_pipeline(pipeline)

    def get_pipeline(self, pipeline_identity) -> Pipeline:
        """
        Description:
            To get an existing pipeline from torch catalog
        :param pipeline_identity: uid or id of the pipeline
        :return:(Pipeline) pipeline class instance
        """
        return self.client.get_pipeline(pipeline_identity)

    def get_pipelines(self):
        """
        Description:
            To get an all pipelines from torch catalog
        :return:(List[PipelineInfo]) pipeline class instance
        """
        return self.client.get_pipelines()

    def get_spans(self, pipeline_run_id):
        """
        Description:
            To get an all spans from torch catalog
        :param pipeline_run_id: run_id of the pipeline
        :return:(List[Span]) pipeline class instance
        """
        return self.client.get_spans(pipeline_run_id)

    def get_pipeline_run(self, pipeline_run_id: str) -> PipelineRun:
        """
        Description:
            To get an existing pipeline from torch catalog
        :param pipeline_run_id: continuation id or run id of the pipeline run
        :return:(PipelineRun) pipeline run class instance
        """
        return self.client.get_pipeline_run(pipeline_run_id)

    def get_pipeline_runs(self, pipeline_id) -> List[PipelineRun]:
        """
        Description:
            To get an all pipeline runs for a pipeline id from torch catalog
        :param pipeline_id: id of the pipeline
        :return:(List[PipelineRun]) List of pipeline run class instance
        """
        return self.client.get_pipeline_runs(pipeline_id)

    def create_datasource(self, datasource: CreateDataSource):
        """
        Description:
            To create datasource in torch catalog
        :param datasource: (CreateDataSource) class instance of the datasource to be created
        :return: (DataSource) newly created datasource
        """
        return self.client.create_datasource(datasource)

    def get_datasource(self, assembly_identifier, properties=False):
        """
        Description:
            Find datasource by it's name or id in torch catalog
        :param assembly_identifier: name or id of the datasource given in torch
        :param properties: optional parameter, bool, to get datasource properties as well
        :return: (DataSource) datasource
        """
        return self.client.get_datasource(assembly_identifier, properties)

    def get_datasource_by_id(self, id: int, properties: bool = False):
        """
        Description:
            Find datasource by its id in torch catalog
        :param id: id of the datasource given in torch
        :return: (DataSource) datasource
        """
        return self.client.get_datasource_by_id(id, properties)

    def get_datasources(self, type: AssetSourceType = None):
        """
        Description:
            Find datasources by its type in torch catalog
        :param type: type of the datasource given in torch, optional
        :return: list(DataSource) datasource
        """
        return self.client.get_datasources(type)

    def get_all_datasources(self):
        """
        Description:
            list all datasources in torch catalog
        :return: (DataSource) list of datasource
        """
        return self.client.get_all_datasources()

    def start_crawler(self, datasource_name: str):
        """
        Description:
            Start crawler for datasource
        :param datasource_name: name of the datasource given in torch
        :return: (CrawlerStatus) CrawlerStatus
        """
        return self.client.start_crawler(datasource_name)

    def get_crawler_status(self, datasource_name: str):
        """
        Description:
            Get crawler status for datasource
        :param datasource_name: name of the datasource given in torch
        :return: (CrawlerStatus) CrawlerStatus
        """
        return self.client.get_crawler_status(datasource_name)

    def get_asset_types(self):
        """
        Description:
            get all asset types supported in torch catalog
        :return: list of asset types
        """
        return self.client.get_all_asset_types()

    def get_profile_status(self, asset_id: int, req_id: int):
        """
        Description:
            get status of asset profile for given request id
        :return: profiling status
        """
        return self.client.get_profile_request_details(asset_id=asset_id, req_id=req_id)

    def get_all_source_types(self):
        """
        Description:
            get all source types supported in torch catalog
        :return: list of all source type
        """
        return self.client.get_all_source_types()

    def get_property_templates(self):
        pass

    def get_connection_types(self):
        """
        Description:
            get all connection types supported in torch catalog
        :return: list of all connection types
        """
        return self.client.get_connection_types()

    def get_tags(self):
        return self.client.get_tags()

    def delete_tag(self, tag: str):
        return self.client.delete_tag(tag)

    def update_data_protection(self, tag: str, is_protected: bool):
        return self.client.update_data_protection(tag, is_protected)

    def discover(self):
        # add filters in discover api & more details in asset class
        return self.client.discover()

    def list_all_snapshots(self):
        return self.client.list_all_snapshots()

    def get_user(self, user_id=None, username=None):
        if user_id is None and username is None:
            raise TorchSdkException('Either provide uid or id to find an asset')
        if user_id is not None:
            return self.client.get_user_by_id(id=user_id)
        if username is not None:
            return self.client.get_user_by_name(username=username)

    def create_api_key(self):
        return self.client.create_api_keys()

    def get_api_keys(self):
        return self.client.get_api_keys()

    def delete_api_key(self, access_key: str):
        return self.client.delete_api_key(access_key)

    def get_notifications(self):
        return self.client.get_notifications()

    def get_incidents(self):
        return self.client.get_incidents()

    def list_all_analysis_pipelines(self):
        return self.client.list_all_analysis_pipelines()

    def get_analysis_pipeline(self, id: int):
        return self.client.get_analysis_pipeline(id)

    def get_all_auto_profile_configurations(self):
        return self.client.get_all_auto_profile_configurations()

    def get_asset(self, identifier):
        """"
            Description:
                Find an asset of the datasource
            :param identifier: uid or ID of the asset
        """
        return self.client.get_asset(identifier)

    def get_policy(self, type: PolicyType, identifier) -> RuleResource:
        """"
            Description:
                Find policy
            :param identifier: ID or name of the policy
            :param type: type of the policy
        """
        if type.value == PolicyType.DATA_QUALITY.value:
            return self.client.get_dq_rule(identifier)
        elif type.value == PolicyType.RECONCILIATION.value:
            return self.client.get_reconciliation_rule(identifier)
        else:
            raise TorchSdkException('Invalid policy type passed.')

    def list_all_policies(self, filter: PolicyFilter, page=0, size=25, withLatestExecution=True, sortBy='updatedAt:DESC'):
        """
        Description:
            To list all policies based on filter
        :param filter: (Enum PolicyFilter) Type of policies to be executed.
        PolicyFilter can have the following parameters
        period: Time period for which list should be filtered. It is an enum ExecutionPeriod
        tags: tags on which list should be filtered
        lastExecutionResult: lastExecution result on which list should be filtered
        asset: Assets for which policies should be filtered. This list should be of objects of type Asset returned by get_asset
        data_source:  Datasource for which policies should be filtered. This list should be of objects of type Datasource
        returned by get_datasource
        policyType: Type of policy policies should be filtered. This is an enum of type RuleType
        enable: To filter only for enabled policies set enable to True
        active: : To filter only for active policies set active to True
        :param page: (Int) page number of query output
        :param size: (Int) Size of each page
        :param sortBy: (Str) sorting order
        """
        return self.client.get_all_rules(filter=filter, page=page, size=size, sortBy=sortBy,
                                         withLatestExecution=withLatestExecution)

    def list_all_classification_configurations(self):
        return self.client.list_all_classification_configurations()

    def create_classification_configuration(self, classification_config):
        return self.client.create_classification_configuration(classification_config)

    def update_classification_configuration(self, classification_config):
        return self.client.update_classification_configuration(classification_config)

    def get_classification_configuration(self, name: str):
        return self.client.get_classification_configuration(name)

    def delete_classification_configuration(self, name: str):
        return self.client.delete_classification_configuration(name)

    def list_setting_groups(self):
        return self.client.list_setting_groups()

    def find_settings_by_group(self, setting_group_name: str):
        return self.client.find_settings_by_group(setting_group_name)

    def get_setting(self, key: str):
        return self.client.get_setting(key)

    def update_setting_value(self, key: str, value: str):
        return self.client.update_setting_value(key, value)

    def reset_setting_value(self, key:str):
        return self.client.reset_setting_value(key)

    def list_all_connections(self):
        return self.client.list_all_connections()

    def create_connection(self, create_connection):
        return self.client.create_connection(create_connection)

    def update_connection(self, update_connection):
        if update_connection.name is None:
            raise TorchSdkException('Update connection should have name of the connection. (update_connection : '
                                    'CreateConnection class instance)')
        return self.client.update_connection(update_connection.name, update_connection)

    def delete_connection(self, id):
        return self.client.delete_connection(id)

    def check_connection(self, connection):
        return self.client.check_connection(connection)

    def get_connection(self, name):
        return self.client.get_connection(name)

    def execute_dq_rule(self, rule_id, incremental=False):
        return self.client.execute_dq_rule(rule_id, incremental)

    def execute_reconciliation_rule(self, rule_id, incremental=False):
        return self.client.execute_reconciliation_rule(rule_id, incremental)

    def get_dq_rule_execution_details(self, execution_id):
        return self.client.get_dq_rule_execution_details(execution_id)

    def get_reconciliation_rule_execution_details(self, execution_id):
        return self.client.get_reconciliation_rule_execution_details(execution_id)

    def cancel_rule_execution(self, execution_id):
        return self.client.cancel_rule_execution(execution_id)

    def enable_rule(self, rule_id):
        return self.client.enable_rule(rule_id)

    def disable_rule(self, rule_id):
        return self.client.disable_rule(rule_id)

    def get_reconciliation_rule_result(self, execution_id) -> RuleResult:
        return self.client.get_reconciliation_rule_result(execution_id)

    def get_dq_rule_result(self, execution_id) -> RuleResult:
        return self.client.get_dq_rule_result(execution_id)

    def execute_policy(self, policy_type: PolicyType, policy_id, sync=True, incremental=False, failure_strategy: FailureStrategy = FailureStrategy.DoNotFail):
        """
        Description:
            To execute policies synchronously and asynchronously
        :param policy_type: (PolicyType) Type of rule to be executed
        :param policy_id: (String) id of the rule to be executed
        :param sync: (bool) optional Set it to False if asynchronous execution has to be done
        :param incremental: (bool) optional Set it to True if full execution has to be done
        :param failure_strategy: (enum) optional Set it to decide if it should fail at error,
            fail at warning or never fail
        """
        return self.client.execute_rule(policy_type, policy_id, sync, incremental, failure_strategy)

    def get_policy_status(self, policy_type, execution_id) -> RuleExecutionStatus:
        """
        Description:
            To get status of rule execution
        :param policy_type: (PolicyType) Type of rule to be executed
        :param execution_id: (String) ID of execution of the rule previously executed.
        """
        return self.client.get_rule_status(policy_type, execution_id)

    def get_policy_execution_result(self, policy_type: PolicyType, execution_id,
                                    failure_strategy: FailureStrategy = FailureStrategy.DoNotFail):
        """
        Description:
            To get result of rule execution
        :param policy_type: (PolicyType) Type of rule to be executed
        :param execution_id: (String) ID of execution of the rule previously executed.
        :param failure_strategy: (enum) optional Set it to decide if it should fail at error,
            fail at warning or never fail
        """
        return self.client.get_rule_execution_result(policy_type, execution_id, failure_strategy)

    def policy_executions(self, identifier, policy_type: RuleType, page=0, size=25, sortBy='finishedAt:DESC'):
        """
        Description:
            To list rule executions
        :param identifier: id or name of executed rule
        :param policy_type: (Enum RuleType) Type of executed rule
        :param page: (Int) page number of query output
        :param size: (Int) Size of each page
        :param sortBy: (Str) sorting order
        """
        return self.client.policy_executions(identifier, policy_type, page, size, sortBy)



import logging
import json
from datetime import datetime
from acceldata_sdk.initialiser import torch_client_credentials
from acceldata_sdk.models.job import CreateJob, JobMetadata, Node
from acceldata_sdk.models.pipeline import CreatePipeline, PipelineMetadata, PipelineRunResult, PipelineRunStatus
from acceldata_sdk.torch_client import TorchClient
from acceldata_sdk.models.span_context import SpanContext

logging.basicConfig(level=logging.INFO)

# Set up acceldata_sdk_code client
torchClient = TorchClient(url="https://torch.acceldata.local:5443/torch",
                       access_key="P04IM8FNQRUCRTU", secret_key="E6LL9YUPMG4BDTJHT2VZD75HW0B8E5")

# create pipeline object
# pipeline = CreatePipeline(
#     uid='monthly_reporting_airflow_2',
#     name='Monthly reporting Pipeline_airflow_2',
#     description='Pipeline to create monthly reporting tables',
#     meta=PipelineMetadata('vaishvik', 'acceldata_sdk_code', '...'),
#     context={'key1': 'value1'}
# )
# creating pipeline using acceldata_sdk_code client
pipelineResponse = torchClient.get_pipeline(pipeline_identity="torch.airflow.demo.lambda")
print('Newly Created Pipeline Response : ', pipelineResponse)

# create job object
# job = CreateJob(
#     uid='monthly_sales_aggregate-38',
#     name='Monthly Sales Aggregate-38',
#     pipelineSnapshot=pipeline.currentSnapshot,
#     description='Generates the monthly sales aggregate tables for the complete year',
#     inputs=[Dataset('postgres-assembly-5450', 'ad_catalog.ad_catalog.flyway_schema_history')],
#     outputs=[Dataset('postgres-ds', 'ad_catalog.ad_catalog.flyway_schema_history')],
#     meta=JobMetadata('vaishvik', 'backend', 'https://github.com/acme/reporting/reporting.scala'),
#     context={'key21': 'value21'}
# )
# # create a job on newly created pipeline
# jobResponse = pipelineResponse.create_job(job)
# print('Newly Created Job Response : ', jobResponse)

# create a pipeline run of the pipeline
pipelineRunResponse = pipelineResponse.create_pipeline_run()
print('pipeline Run Created :', pipelineRunResponse)

# update a pipeline run of the pipeline
# updatePipelineRunRes = pipelineRunResponse.update_pipeline_run(context_data={'key1': 'value2', 'name': 'backend'},
#                                                                result=PipelineRunResult.SUCCESS,
#                                                                status=PipelineRunStatus.COMPLETED)
# print('pipeline run updated :', updatePipelineRunRes)

# create span in the pipeline run
spanContext = pipelineRunResponse.create_span(uid='span_uid_38')
print('Span Created :', spanContext)
child_span_context = spanContext.create_child_span(
    uid="test.child.span",
    context_data={
        'time': str(datetime.now())
    })
# span_dict = child_span_context.get_dict()
# json = json.dumps(span_dict).encode('UTF-8')
# SpanContext(**span_dict)


# # start span event
# print('Span Start Event Response :', spanContext.start({'k': 'v'}))

# check current span is root or not
# print('span is root or not : ', spanContext.is_root())
#
# # get span
# sc = pipelineRunResponse.get_span(span_uid='span_uid_38')
# print('span con get : ', sc)
#
# sc2 = pipelineRunResponse.get_span(span_uid='span_uid_38')
# print('span con get : ', sc2)
#
# # end the span event
# print('Span End Event Response :', spanContext.end())
#
# # check if the current span has children or not
# print('has child ', spanContext.has_children())

# # create a child span
# childSpanContext = spanContext.create_child_span('span_uid_38_2')
# print('child span is root or not  ', childSpanContext.is_root())
# print('child span abort event : ', childSpanContext.abort())
#
# print('child span has child or not : ', childSpanContext.has_children())
# print('parent span has child or not : ', spanContext.has_children())


from enum import Enum, auto
from dataclasses import dataclass, asdict
from typing import List

from acceldata_sdk.models.ruleExecutionResult import RuleExecutionResult, RuleItemResult, RuleExecutionSummary, RuleResult
from acceldata_sdk.models.ruleExecutionResult import ExecutorConfig
from acceldata_sdk.models.rule import ShortSegment, Label, BackingAsset, RuleThresholdLevel, RuleTag, PolicyGroup, RuleResource, RuleExecution
from acceldata_sdk.constants import FailureStrategy, PolicyType


class ReconRuleExecutionSummary(RuleExecutionSummary):
    def __init__(self, ruleId, executionMode, executionStatus, resultStatus, startedAt, executionType,
                 isProtectedResource, thresholdLevel, ruleVersion, id=None, ruleName=None, ruleType=None,
                 lastMarker=None, leftLastMarker=None, rightLastMarker=None, executionError=None, finishedAt=None,
                 resetPoint=None, persistencePath=None, resultPersistencePath=None, executorConfig=None,
                 markerConfig=None, leftMarkerConfig=None, rightMarkerConfig=None, dataPersistenceEnabled=None,
                 **kwargs):
        super().__init__(ruleId, executionMode, executionStatus, resultStatus, startedAt,executionType, thresholdLevel,
                         ruleVersion, id, ruleName, ruleType, lastMarker, leftLastMarker, rightLastMarker,
                         executionError, finishedAt, resetPoint, persistencePath, resultPersistencePath, executorConfig,
                         markerConfig, leftMarkerConfig, rightMarkerConfig, dataPersistenceEnabled, isProtectedResource)

    def __repr__(self):
        return f"ReconRuleExecutionSummary({self.__dict__})"


class ReconcillationRuleExecutionResult(RuleExecutionResult):
    def __init__(self, status, description=None, successCount=None, failureCount=None, leftRows=None, rightRows=None,
                 qualityScore=None, **kwargs):
        super().__init__(status, description, successCount, failureCount, qualityScore)
        self.leftRows = leftRows
        self.rightRows = rightRows

    def __repr__(self):
        return f"ReconcillationRuleExecutionResult({self.__dict__})"


class MappingOperation(Enum):
    EQ = auto()
    NOT_EQ = auto()
    GTE = auto()
    GT = auto()
    LTE = auto()
    LT = auto()


class ColumnMapping:
    def __init__(self, leftColumnName, operation, rightColumnName, useForJoining, isJoinColumnUsedForMeasure,
                 ignoreNullValues, weightage, ruleVersion,isWarning, businessExplanation=None, id=None,
                 reconciliationRuleId=None, deletedAt=None, labels=None, **kwargs):
        self.id = id
        self.leftColumnName = leftColumnName
        if isinstance(operation, dict):
            self.operation = MappingOperation(**operation)
        else:
            self.operation = operation
        self.rightColumnName = rightColumnName
        self.useForJoining = useForJoining
        self.isJoinColumnUsedForMeasure = isJoinColumnUsedForMeasure
        self.ignoreNullValues = ignoreNullValues
        self.weightage = weightage
        self.ruleVersion = ruleVersion
        self.isWarning = isWarning
        self.businessExplanation = businessExplanation
        self.reconciliationRuleId = reconciliationRuleId
        self.deletedAt = deletedAt
        self.labels = list()
        for obj in labels:
            if isinstance(obj, dict):
                self.labels.append(Label(**obj))
            else:
                self.labels.append(obj)

    def __repr__(self):
        return f"ColumnMapping({self.__dict__})"


class ReconcillationRuleItemResult(RuleItemResult):
    def __init__(self, id, ruleItemId, threshold, weightage, isRowMatchMeasure, isWarning, columnMapping=None,
                 resultPercent=None, success=None, error=None, **kwargs):
        super().__init__(id, ruleItemId, threshold, weightage, isWarning, resultPercent, success, error)
        self.isRowMatchMeasure = isRowMatchMeasure
        if isinstance(columnMapping, dict):
            self.columnMapping = ColumnMapping(**columnMapping)
        else:
            self.columnMapping = columnMapping

    def __repr__(self):
        return f"ReconcillationRuleItemResult({self.__dict__})"


class ReconcillationExecutionResult(RuleResult):
    def __init__(self, execution, items, meta=None, result=None, **kwargs):
        if isinstance(execution, dict):
            self.execution = ReconRuleExecutionSummary(**execution)
        else:
            self.execution = execution
        if isinstance(result, dict):
            self.result = ReconcillationRuleExecutionResult(**result)
        else:
            self.result = result
        self.items = list()
        for obj in items:
            if isinstance(obj, dict):
                self.items.append(ReconcillationRuleItemResult(**obj))
            else:
                self.items.append(obj)
        self.meta = meta
        self.executionId = self.execution.id

    def __repr__(self):
        return f"ReconcillationExecutionResult({self.__dict__})"


class ReconciliationMeasurementType(Enum):
    EQUALITY = 'EQUALITY'
    COUNT = 'COUNT'
    PROFILE_EQUALITY = 'PROFILE_EQUALITY'
    HASHED_EQUALITY = 'HASHED_EQUALITY'
    TIMELINESS = 'TIMELINESS'
    CUSTOM = 'CUSTOM'


@dataclass
class Item:
    id = None
    ruleId = None
    measurementType = None
    executionOrder = None
    version = None

    def __init__(self,
                 id=None,
                 ruleId=None,
                 measurementType = None,
                 executionOrder = None,
                 version = None
                 ):
        self.ruleId = ruleId
        self.id = id
        self.executionOrder = executionOrder
        self.version = version
        if isinstance(measurementType, dict):
            self.measurementType = ReconciliationMeasurementType(**measurementType)
        else:
            self.measurementType = measurementType


class ReconciliationRuleDetails:
    def __init__(self, ruleId, leftBackingAssetId, rightBackingAssetId, columnMappings, timeSecondsOffset,
                 items, isSegmented, segments, id=None, **kwargs):
        self.id = id
        self.ruleId = ruleId
        self.leftBackingAssetId = leftBackingAssetId
        self.rightBackingAssetId = rightBackingAssetId
        self.timeSecondsOffset = timeSecondsOffset
        self.items = list()
        for obj in items:
            if isinstance(obj, dict):
                self.items.append(Item(**obj))
            else:
                self.items.append(obj)
        self.isSegmented = isSegmented
        self.segments = list()
        if segments is not None:
            for obj in segments:
                if isinstance(obj, dict):
                    self.segments.append(ShortSegment(**obj))
                else:
                    self.segments.append(obj)
        self.columnMappings = list()
        if columnMappings is not None:
            for obj in columnMappings:
                if isinstance(obj, dict):
                    self.columnMappings.append(ColumnMapping(**obj))
                else:
                    self.columnMappings.append(obj)

    def __repr__(self):
        return f"ReconciliationRuleDetails({self.__dict__})"


class ChannelType(Enum):
    JIRA = 'JIRA'
    EMAIL = 'EMAIL'
    SLACK = 'SLACK'
    HANGOUT = 'HANGOUT'
    WEBHOOK = 'WEBHOOK'


class NotifyOn(Enum):
    ERROR = 'ERROR'
    WARNING = 'WARNING'
    SUCCESS = 'SUCCESS'
    ALL = 'ALL'


@dataclass
class NotificationPayload:
    configuredList: List
    notifyOn: List[NotifyOn]
    tags: List[str] = None


class ReconciliationRule:
    def __init__(self, name, description, type, enabled, schedule,scheduled,notificationChannels, leftBackingAsset,
                 rightBackingAsset,timeSecondsOffset, createdAt, updatedAt, thresholdLevel, archived, archivalReason,
                 tenantId, createdBy, lastUpdatedBy, tags, version, executorConfig, labels, isProtectedResource,
                 policyGroups, id=None, **kwargs):
        self.id = id
        self.name = name
        self.description = description
        self.type = type
        self.enabled = enabled
        self.schedule = schedule
        self.scheduled = scheduled
        self.notificationChannels = notificationChannels
        if isinstance(leftBackingAsset, dict):
            self.leftBackingAsset = BackingAsset(**leftBackingAsset)
        else:
            self.leftBackingAsset = leftBackingAsset
        if isinstance(rightBackingAsset, dict):
            self.rightBackingAsset = BackingAsset(**rightBackingAsset)
        else:
            self.rightBackingAsset = rightBackingAsset
        self.timeSecondsOffset = timeSecondsOffset
        self.createdAt = createdAt
        self.updatedAt = updatedAt
        if isinstance(thresholdLevel, dict):
            self.thresholdLevel = RuleThresholdLevel(**thresholdLevel)
        else:
            self.thresholdLevel = thresholdLevel
        self.archived = archived
        self.archivalReason = archivalReason
        self.tenantId = tenantId
        self.createdBy = createdBy
        self.lastUpdatedBy = lastUpdatedBy

        self.tags = list()
        if tags is not None:
            for obj in tags:
                if isinstance(obj, dict):
                    self.tags.append(RuleTag(**obj))
                else:
                    self.tags.append(obj)
        self.version = version
        if isinstance(executorConfig, dict):
            self.executorConfig = ExecutorConfig(**executorConfig)
        else:
            self.executorConfig = executorConfig
        self.labels = list()
        if labels is not None:
            for obj in labels:
                if isinstance(obj, dict):
                    self.labels.append(Label(**obj))
                else:
                    self.labels.append(obj)
        self.isProtectedResource = isProtectedResource

        self.policyGroups = list()
        if policyGroups is not None:
            for obj in policyGroups:
                if isinstance(obj, dict):
                    self.policyGroups.append(PolicyGroup(**obj))
                else:
                    self.policyGroups.append(obj)

    def __repr__(self):
        return f"ReconciliationRule({self.__dict__})"


class ReconciliationRuleResource(RuleResource):
    def __init__(self, rule, details, client=None, **kwargs):
        if isinstance(rule, dict):
            self.rule = ReconciliationRule(**rule)
        else:
            self.rule = rule
        if isinstance(details, dict):
            self.details = ReconciliationRuleDetails(**details)
        else:
            self.details = details
        self.client = client

    def execute(self, sync=True, incremental=False, failure_strategy: FailureStrategy = FailureStrategy.DoNotFail) -> RuleExecution:
        return self.client.execute_rule(PolicyType.RECONCILIATION, self.rule.id, sync, incremental, failure_strategy)

    def get_executions(self, page=0, size=25, sortBy='finishedAt:DESC'):
        return self.client.policy_executions(self.rule.id, PolicyType.RECONCILIATION, page, size, sortBy)

    def __repr__(self):
        return f"ReconciliationRuleResource({self.__dict__})"



from abc import abstractmethod

from acceldata_sdk.models.create_asset import CreateAsset
from acceldata_sdk.models.datasource import DataSource
from acceldata_sdk.models.pipeline import Pipeline
from acceldata_sdk.models.job import Job


class TorchClientInterFace:
    """
    Description: TorchClientInterface is the interface that contains different methods that are supported to
    communicate with catalog-server
    """

    @abstractmethod
    def create_pipeline(self, pipeline: Pipeline): raise NotImplementedError

    @abstractmethod
    def delete_pipeline(self, pipeline: Pipeline): raise NotImplementedError

    @abstractmethod
    def create_job(self, job: {}, pipelineId: int): raise NotImplementedError

    @abstractmethod
    def delete_job(self, job: Job = None): raise NotImplementedError

    @abstractmethod
    def create_datasource(self, datasource: DataSource): raise NotImplementedError

    @abstractmethod
    def create_asset(self, asset: CreateAsset): raise NotImplementedError

    @abstractmethod
    def initialise_snapshot(self, snapshot_data): raise NotImplementedError

    @abstractmethod
    def get_current_snapshot(self, assembly_id: int): raise NotImplementedError

    @abstractmethod
    def create_asset_relation(self, asset_relation): raise NotImplementedError

    @abstractmethod
    def create_pipeline_run(self, pipeline_run): raise NotImplementedError

    @abstractmethod
    def update_pipeline_run(self, pipeline_run_id: int, pipeline_run): raise NotImplementedError

    @abstractmethod
    def create_span(self, pipeline_run_id: int, span: dict): raise NotImplementedError

    @abstractmethod
    def create_span_event(self, span_event): raise NotImplementedError



import logging

from acceldata_sdk.initialiser import torch_client_credentials
from acceldata_sdk.models.job import CreateJob, JobMetadata, Dataset
from acceldata_sdk.models.pipeline import CreatePipeline, PipelineMetadata, PipelineRunResult, PipelineRunStatus
from acceldata_sdk.torch_client import TorchClient

logging.basicConfig(level=logging.INFO)

# Set up acceldata_sdk_code client
torchClient = TorchClient(url="https://torch.acceldata.local:5443",
                       access_key="N1LTYRK630PZ", secret_key="xPeUj4Iyj4WL2Tw284s9mqsgxvbPKW")

# get pipeline object
pipeline = torchClient.get_pipeline('customer.orders.monthly.agg.demo')
print('pipeline response : ', pipeline)

# # create job object
pipeline_run = pipeline.get_latest_pipeline_run()
job = CreateJob(
    uid='monthly_sales_aggregate-39',
    name='Monthly Sales Aggregate-39',
    version=pipeline_run.versionId,
    description='Generates the monthly sales aggregate tables for the complete year',
    inputs=[Dataset('postgres-assembly-5450', 'ad_catalog.ad_catalog.flyway_schema_history')],
    outputs=[Dataset('postgres-ds', 'ad_catalog.ad_catalog.flyway_schema_history')],
    meta=JobMetadata('Vaishvik_brahmbhatt', 'backend', 'https://github.com/acme/reporting/report.scala'),
    context={'job_time': 'weekend'}
)
# create a job on newly created pipeline
jobResponse = pipeline.create_job(job)
print('Newly Created Job Response : ', jobResponse)

# get latest pipeline run
pipeline_run = pipeline.get_latest_pipeline_run()
print('latest pipeline run : ', pipeline_run)

# create span in the pipeline run
span_context = pipeline_run.get_span(span_uid='main_span')
print('Span get :', span_context)

# end the span event
print('Span End Event Response :', span_context.end())

# check if the current span has children or not
print('has child ', span_context.has_children())

# # create a child span
# print('Created child span .........................')
#
# childSpanContext = span_context.create_child_span(uid='span_uid_39_3', context_data={'span': 'child', 'end': 'wednesday'})
# print('child span is root or not  ', childSpanContext.is_root())
# print('child span abort event : ', childSpanContext.abort())
# print('child span abort event : ', childSpanContext.abort())
# print('child span has child or not : ', childSpanContext.has_children())
#
# # update a pipeline run of the pipeline
# update_pipeline_run = pipeline_run.update_pipeline_run(context_data={'key1': 'value2', 'name': 'backend'},
#                                                        result=PipelineRunResult.SUCCESS,
#                                                        status=PipelineRunStatus.COMPLETED)
# print('pipeline run updated :', update_pipeline_run)





class SettingGroup:

    def __init__(self, id, name, displayName, description, **kwargs):
        self.id = id
        self.name = name
        self.displayName = displayName
        self.description = description

    def __eq__(self, other):
        return self.id == other.id

    def __repr__(self):
        return f"Setting-Group({self.__dict__})"


class Setting:

    def __init__(self, id, key, displayName, value, defaultValue, isReadOnly: bool, valueType, settingGroupId, description = None, **kwargs):
        self.id = id
        self.key = key
        self.displayName = displayName
        self.value = value
        self.defaultValue = defaultValue
        self.isReadOnly = isReadOnly
        self.valueType = valueType
        self.settingGroupId = settingGroupId
        self.description = description

    def __eq__(self, other):
        return self.id == other.id

    def __repr__(self):
        return f"Setting({self.__dict__})"


import psycopg2
import random
import datetime
import time

from acceldata_sdk.models.job import CreateJob, JobMetadata, Dataset
from acceldata_sdk.events.generic_event import GenericEvent
from acceldata_sdk.models.pipeline import CreatePipeline, PipelineMetadata, PipelineRunResult, PipelineRunStatus
from acceldata_sdk.torch_client import TorchClient


def create_torch_client():
    return TorchClient(url='https://acceldata.mars.acceldata.dev/torch', access_key = '8V4CBJZZL70QTC7', secret_key ='NM4L5ESV9F65SN5AP0C7L8UUAE1H01')


def create_pipeline(torch_client):
    pipeline = CreatePipeline(
        uid='customer.orders.monthly',
        name='Customer Orders Monthly aggregate',
        description='Pipeline to Aggregate the customer orders over 1 year',
        meta=PipelineMetadata(
            owner='vaishvik', team='acceldata_sdk_code', codeLocation='...'),
        context={'associated_tables': 'pipeline.customer, pipeline.orders, pipeline.customer_orders, pipeline.customer_orders_monthly_agg'}
    )
    pipeline_response = torch_client.create_pipeline(pipeline=pipeline)
    print('Created the pipeline')
    return pipeline_response


def create_datagen_job(pipeline):
    pipeline_run = pipeline.get_latest_pipeline_run()
    job = CreateJob(
        uid='customer.order.datagen.job',
        name='customers and orders datagen',
        version=pipeline_run.versionId,
        description='Generates Pseudo random data for Orders and Customers',
        inputs=[],
        outputs=[
            Dataset('snowflake1647503515210',
                    'FINANCE.FINANCE.A1'), Dataset('snowflake1647503515210',
                                                         'FINANCE.FINANCE.A2')],
        meta=JobMetadata(owner='vaishvik', team='backend',
                         codeLocation='https://github.com/acme/reporting/reporting.scala'),
        context={}
    )
    job = pipeline.create_job(job)
    print('Created Job for random data insertion')
    return job


def create_data_join_job(pipeline):
    pipeline_run = pipeline.get_latest_pipeline_run()
    job = CreateJob(
        uid='customer.order.join.job',
        name='customers and orders joiner and Inserter',
        version=pipeline_run.versionId,
        description='Joins the ',
        inputs=[Dataset('snowflake1647503515210',
                        'FINANCE.FINANCE.A1'), Dataset('snowflake1647503515210',
                                                             'FINANCE.FINANCE.A2')],
        outputs=[
            Dataset('snowflake1647503515210',
                    'FINANCE.FINANCE.A3')],
        meta=JobMetadata(owner='vaishvik', team='backend',
                         codeLocation='https://github.com/acme/reporting/reporting.scala'),
        context={}
    )
    job = pipeline.create_job(job)
    print('Created Job for joined data read and insert')
    return job


def create_pipeline_run(pipeline):
    return pipeline.create_pipeline_run(
        context_data={'client_time': str(datetime.datetime.now())})


def start_main_span(pipeline_run):
    span_context = pipeline_run.create_span(
        uid='customer.orders.monthly.agg', context_data={'client_time': str(datetime.datetime.now())})
    return span_context


def end_main_span(span_context):
    span_context.end()


def end_pipeline_run(pipeline_run, result=PipelineRunResult.SUCCESS, status=PipelineRunStatus.COMPLETED):
    pipeline_run.update_pipeline_run(context_data={'client_time': str(datetime.datetime.now())},
                                     result=result,
                                     status=status)


def create_data_agg_job(pipeline):
    job = CreateJob(
        uid='monthly.order.aggregate.job',
        name='Aggregates the monthly orders',
        description='Joins the ',
        inputs=[Dataset('snowflake1647503515210',
                        'FINANCE.FINANCE.A3')],
        outputs=[
            Dataset('snowflake1647503515210',
                    'FINANCE.FINANCE.A4')],
        meta=JobMetadata(owner='vaishvik', team='backend',
                         codeLocation='https://github.com/acme/reporting/reporting.scala'),
        context={}
    )
    job = pipeline.create_job(job)
    print('Created Job for monthly aggregation')
    return job


def create_conn():
    return None
    # return psycopg2.connect(
    #     host="torch.acceldata.local",
    #     port=5433,
    #     database="example",
    #     user="postgres",
    #     password="postgres")


def create_customer(index):
    return "customer" + str(index)


def create_orders(customer_id):
    orders = []
    for i in range(5):
        add_months = random.randint(0, 11)
        order = {
            'customer_id': customer_id,
            'items_count': random.randint(5, 10),
            'unit_price': random.randint(10, 20),
            'created_at': datetime.datetime(2021, 1, 1) +
            datetime.timedelta(days=30 * add_months)
        }
        orders.append(order)
    return orders


def insert_data(conn, span_context):
    customer_sql = """INSERT INTO pipeline.customers(name) VALUES(%s) RETURNING id;"""
    orders_sql = """INSERT INTO pipeline.orders(customer_id, items_count, unit_price, created_at) VALUES(%s, %s, %s, %s) RETURNING id;"""
    datagen_span_context = span_context.create_child_span(
        uid="customer.orders.datagen.span", context_data={'client_time': str(datetime.datetime.now())})
    customer_datagen_span = datagen_span_context.create_child_span(
        uid="customer.data.gen", context_data={'client_time': str(datetime.datetime.now())})

    # cur = conn.cursor()
    # customer_ids = []
    # print("Creating dummy customers")
    # for i in range(1, 101):
    #     customer_name = create_customer(i)
    #     cur.execute(customer_sql, (customer_name,))
    #     customer_id = cur.fetchone()[0]
    #     customer_ids.append(customer_id)
    # conn.commit()

    time.sleep(2)

    customer_datagen_span.end(
        {'client_time': str(datetime.datetime.now()), 'customers_count': 100})

    print("Creating dummy orders for each customers")
    orders_datagen_span = datagen_span_context.create_child_span(
        uid="order.data.gen", context_data={'client_time': str(datetime.datetime.now())})

    # cur = conn.cursor()
    # count = 0
    # for customer_id in customer_ids:
    #     customer_orders = create_orders(customer_id)
    #     for order in customer_orders:
    #         cur.execute(orders_sql, (customer_id,
    #                                  order['items_count'], order['unit_price'], order['created_at']))
    #         count += 1
    #
    # cur.close()
    # conn.commit()

    time.sleep(2)
    orders_datagen_span.end(
        {'client_time': str(datetime.datetime.now()), 'orders_count': 100})
    datagen_span_context.end({'client_time': str(datetime.datetime.now())})


def execute_join_orders_query(conn, span_context):
    join_sql = """SELECT co.id as customer_id, co.name, o.items_count * o.unit_price as total_order_value, o.created_at as ordered_at
    FROM pipeline.orders o JOIN pipeline.customers co on o.customer_id = co.id;"""

    join_span_context = span_context.create_child_span(
        uid="customer.orders.join.span", context_data={'client_time': str(datetime.datetime.now())})
    print("Joining customers and orders")
    # cur = conn.cursor()
    # cur.execute(join_sql)
    # rows = cur.fetchall()
    # cur.close()
    # conn.commit()
    # time.sleep(2)
    join_span_context.send_event(GenericEvent(context_data={'client_time': str(
        datetime.datetime.now()), 'row_count': 100}, event_uid="order.customer.join.result"))
    join_span_context.end(
        {'client_time': str(datetime.datetime.now()), 'row_count': 100})
    return None


def insert_orders(conn, rows, span_context):
    insert_sql = """INSERT INTO pipeline.customer_orders(customer_id, customer_name, total_order_value, ordered_at)
             VALUES(%s, %s, %s, %s) RETURNING id;"""
    print("Inserting joined record into the orders table")
    insert_span_context = span_context.create_child_span(
        uid="customer.orders.insert.span", context_data={'client_time': str(datetime.datetime.now())})
    # cur = conn.cursor()
    # for row in rows:
    #     cur.execute(insert_sql, (row[0], row[1], row[2], row[3]))
    # cur.close()
    # conn.commit()
    time.sleep(2)
    insert_span_context.end({'client_time': str(datetime.datetime.now())})


def insert_aggregate_data(conn, span_context):
    insert_sql = """
      INSERT INTO pipeline.customer_orders_monthly_agg(customer_name, customer_id, order_month, total_order_value)
      SELECT customer_name, customer_id,EXTRACT(MONTH FROM ordered_at) as month, SUM(total_order_value)
      FROM pipeline.customer_orders GROUP BY customer_name, EXTRACT(MONTH FROM ordered_at), customer_id;
    """
    print("Aggregating the orders over months per customer")
    agg_span_context = span_context.create_child_span(
        uid="monthy.aggregate.span", context_data={'client_time': str(datetime.datetime.now())})
    # cur = conn.cursor()
    # cur.execute(insert_sql)
    # cur.close()
    # conn.commit()
    time.sleep(2)
    agg_span_context.end({'client_time': str(datetime.datetime.now())})


if __name__ == "__main__":
    torch_client = create_torch_client()
    conn = create_conn()

    pipeline = create_pipeline(torch_client)
    pipeline_run = create_pipeline_run(pipeline)
    create_datagen_job(pipeline)
    create_data_join_job(pipeline)
    create_data_agg_job(pipeline)

    span_context = start_main_span(pipeline_run)
    insert_data(conn, span_context)
    time.sleep(2)
    rows = execute_join_orders_query(conn, span_context)
    time.sleep(2)
    insert_orders(conn, None, span_context)
    time.sleep(2)
    insert_aggregate_data(conn, span_context)
    time.sleep(2)
    # conn.close()
    end_main_span(span_context)
    end_pipeline_run(pipeline_run)



from dataclasses import dataclass, asdict
from typing import List, Dict
from enum import Enum, auto

from acceldata_sdk.errors import TorchSdkException
from acceldata_sdk.models.snapshot import SnapshotData, AssociatedItemType
from acceldata_sdk.models.create_asset import CreateAsset, CreateAssetRelation, RelationType, AssetMetadata
from acceldata_sdk.models.connection import Connection


@dataclass
class ConfigProperty:
    key=None
    value = None

    def __init__(self,
                 key=None,
                 value=None):
        self.key = key
        self.value = value


@dataclass
class Crawler:
    name = None

    def __init__(self, name=None):
        self.name = name

    def __repr__(self):
        return f"Crawler({self.__dict__})"


@dataclass
class DatasourceType:
    id = None
    name = None

    def __init__(self,
                 id = None,
                 name=None):
        self.name = name
        self.id = id


class CreateDataSource:

    def __init__(self,
                 name: str,
                 sourceType: DatasourceType,
                 description: str = None,
                 isVirtual: bool = None,
                 connectionId: int = None,
                 configProperties: List[ConfigProperty] = []
                 ):
        self.name = name
        self.description = description
        self.sourceType = DatasourceType(sourceType.id, sourceType.name)
        if isVirtual is None and connectionId is None:
            raise TorchSdkException('Either provide connection configuration for the assembly or enable isVirtual flag')
        if isVirtual is None:
            self.connectionId = connectionId
            self.configProperties = configProperties
            self.isVirtual = False
        if connectionId is None:
            self.isVirtual = isVirtual

    def __eq__(self, other):
        return self.name == other.name and self.connectionId == other.connectionId

    def __repr__(self):
        return f"DataSource({self.name!r})"


@dataclass
class DatasourceSourceModel:
    id = None
    name = None

    def __init__(self,
                 id = None,
                 name=None):
        self.name = name
        self.id = id


@dataclass
class DatasourceSourceType:

    def __init__(self, id, name, sourceModel=None, connectionTypeId=None, **kwargs):
        """
            Description:
                Datasource source type
        :param id: id of the source type
        :param name: name of the source type
        :param sourceModel: source model
        :param connectionTypeId: (int) connection type id for the given source type
        """
        self.id = id
        self.name = name
        self.connectionTypeId = connectionTypeId
        if isinstance(sourceModel, dict):
            self.sourceModel = DatasourceSourceModel(**sourceModel)
        else:
            self.sourceModel = sourceModel

    def __repr__(self):
        return f"DatasourceSourceType({self.__dict__})"


class ChildType:

    def __init__(self, canProfile, canSample, id, name):
        self.canProfile = canProfile
        self.canSample = canSample
        self.id = id
        self.name = name

    def __eq__(self, other):
        return self.id == other.id

    def __repr__(self):
        return f"ChildType({self.__dict__})"


class RootAsset:

    def __init__(self, assetId=None, alias=None, isCustom=False, childType=None, name=None, parentId=None):
        self.assetId = assetId
        self.alias = alias
        self.isCustom = isCustom
        if isinstance(childType, dict):
            self.datasource = ChildType(**childType)
        else:
            self.datasource = childType
        self.name = name
        self.parentId = parentId

    def __repr__(self):
        return f"DataSourceRootAsset({self.__dict__})"


class PropertyType(Enum):
    INT = auto()
    FLOAT = auto()
    STRING = auto()
    BOOLEAN = auto()
    JSON = auto()
    DURATION = auto()
    OBJECT = auto()
    OPTIONS = auto()
    URL = auto()
    URI = auto()
    PASSWORD = auto()
    EMAIL = auto()
    CRONEXPRESSION = auto()
    FILE = auto()
    ARRAY = auto()

    class SubPropertyType(Enum):
        STRING = auto()
        INT = auto()
        FLOAT = auto()
        BOOLEAN = auto()


class OptionTypes(Enum):

    CSV = auto()
    PARQUET = auto()
    ORC = auto()
    JSON = auto()
    AVRO = auto()
    CONFLUENT_AVRO = auto()

    PLAINTEXT = auto()
    SASL_PLAINTEXT = auto()
    SASL_SSL = auto()
    SSL = auto()
    BASIC_AUTH = auto()

    STANDALONE = auto()
    ORACLE_CLOUD_AUTONOMOUS_DB = auto()

    # Postgresql Environments
    ON_PREMISE = auto()
    AMAZON_RDS = auto()

    # Analysis Service Data Processor types
    HADOOP = auto()
    KUBERNETES = auto()
    DATABRICKS = auto()

    # Kafka Schema Naming Strategies
    TOPIC_NAME = auto()
    RECORD_NAME = auto()
    TOPIC_RECORD_NAME = auto()
    KEY = auto()
    VALUE = auto()

    # Torch Result Location Types
    HDFS = auto()
    S3 = auto()

@dataclass
class PropertyTemplate:
    key: str = None
    description: str = None
    displayLabel: str = None
    type: PropertyType = None
    options: List[OptionTypes] = None
    subType: PropertyType.SubPropertyType = None
    required: bool = None
    children: object = None
    visibility: str = None
    value: str = None
    replacementFor: str = None
    hidden: bool = None
    readOnly: bool = None
    configKey: str = None
    configValue: str = None
    multiSelect: bool = None


@dataclass
class PropertyTemplates:
    connections: Dict[str, List[PropertyTemplate]] = None
    assemblies: Dict[str, List[PropertyTemplate]] = None



@dataclass
class SourceModel:
    id = None,
    name = None

    def __init__(self,
                 id = None,
                 name=None):
        self.name = name
        self.id = id


@dataclass
class SourceType:
    id: str = None
    name: str = None
    sourceModel: SourceModel = None
    connectionTypeId: str = None
    miniProfiling: bool = None
    propertyTemplates: PropertyTemplates = None


@dataclass
class SecurityConfig:
    type = None
    config = None

    def __init__(self,
                 type = None,
                 config=None):
        self.type = type
        self.config = config


class DataSource:

    def __init__(self,
                 name: str = None,
                 isSecured: bool = None,
                 isVirtual: bool = None,
                 id: int = None,
                 createdAt: str = None,
                 updatedAt: str = None,
                 assemblyProperties = None,
                 conn: Connection = None,
                 connectionId: int = None,
                 crawler: Crawler = None,
                 currentSnapshot: str = None,
                 description: str = None,
                 sourceType: SourceType = None,
                 securityConfig: SecurityConfig = None,
                 schedule: str=None,
                 configuration=None,
                 client=None,
                 autoProfile: bool = None,
                 tenantId: str = None,
                 integrationId: str = None,
                 subIntegrationId: str = None,
                 isProtectedResource: bool = None,
                 createdBy: str = None,
                 **kwargs
                 ):
        """
            Description:
                datasource class.
        :param name: name of the datasource
        :param isSecured: is secured or not
        :param isVirtual: is virtual datasource or not
        :param id: id of the datasource
        :param createdAt: creation time of the datasource
        :param updatedAt: updated time of the datasource
        :param assemblyProperties: datasource properties
        :param conn: connection details for the ds
        :param connectionId: connection id of the datasource
        :param crawler: crawler details of the datasource
        :param currentSnapshot: current version of the datasource
        :param description: desc of the datasource
        :param sourceType: (DatasourceSourceModel) source type details
        :param securityConfig: security configuration for the given ds
        :param schedule: scheduled exp
        :param configuration: configurations
        """
        self.name = name
        self.isSecured = isSecured
        self.isVirtual = isVirtual
        self.id = id
        self.createdAt = createdAt
        self.updatedAt = updatedAt
        self.assemblyProperties = assemblyProperties
        if isinstance(conn, dict):
            self.conn = Connection(**conn)
        else:
            self.conn = conn
        self.connectionId = connectionId
        if isinstance(crawler, dict):
            self.crawler = Crawler(**crawler)
        else:
            self.crawler = crawler
        self.currentSnapshot = currentSnapshot
        self.description = description
        if isinstance(securityConfig, dict):
            self.securityConfig = SecurityConfig(**securityConfig)
        else:
            self.securityConfig = securityConfig
        self.schedule = schedule
        self.configuration = configuration
        if isinstance(sourceType, dict):
            self.sourceType = SourceType(**sourceType)
        else:
            self.sourceType = sourceType
        self. autoProfile = autoProfile
        self.tenantId = tenantId
        self.integrationId = integrationId
        self.subIntegrationId = subIntegrationId
        self.isProtectedResource = isProtectedResource
        self.createdBy = createdBy

        self.client = client

    def __repr__(self):
        return f"DataSource({self.__dict__})"

    def update_datasource(self, create_datasource=CreateDataSource):
        return self.client.update_datasource(datasource_id=self.id, datasource=create_datasource)

    def get_root_assets(self):
        return self.client.get_root_asset(self.id)

    # convert asset to dict type
    def _convert_asset_to_dict(self, asset: CreateAsset):
        """
            Description:
                Convert CreateAsset class instance to dict type
            :param asset: CreateAsset class instance
            :return: dict form of CreateAsset class instance
        """
        payload = asset.__dict__
        metadata = []
        for md in asset.metadata:
            metadata.append(asdict(md))
        payload['metadata'] = metadata
        asset_payload = {'data': payload}
        return asset_payload

    # to create an asset
    def create_asset(self, name: str, uid: str, asset_type_id: int, parent_id: int = None, description: str = None,
                     snapshots=None, metadata: List[AssetMetadata] = None):
        """
        Description:
            used to create asset in datasource
        :return: asset created
        """
        if snapshots is None:
            snapshots = [self.currentSnapshot]
        asset = CreateAsset(
            name=name,
            description=description,
            assemblyId=self.id,
            uid=uid,
            assetTypeId=asset_type_id,
            sourceTypeId=self.sourceType.id,
            isCustom=False,
            parentId=parent_id,
            currentSnapshot=self.currentSnapshot,
            snapshots=snapshots,
            metadata=metadata
        )
        payload = self._convert_asset_to_dict(asset)
        return self.client.create_asset(payload)

    # convert snapshot data to dict type
    def _convert_snapshot_data_to_dict(self, snapshot_data: SnapshotData):
        """
            Description:
                Convert SnapshotData class instance to dict type
            :param snapshotData: SnapshotData class instance
            :return: dict form of SnapshotData class instance
        """
        payload = snapshot_data.__dict__
        payload['associatedItemType'] = snapshot_data.associatedItemType.name
        snaoshot_payload = {'data': payload}
        return snaoshot_payload

    # initialise new version of snapshot for a datasource
    def initialise_snapshot(self, uid: str):
        """
        Description:
            Used to initialise new version of snapshot for a datasource
        :param uid: uid of new snapshot version
        :return: created snapshotData class instance
        """
        if uid is None:
            raise TorchSdkException('uid for new snapshot version is required')
        snapshot_data = SnapshotData(
            uuid=uid,
            associatedItemType=AssociatedItemType.ASSEMBLY,
            associatedItemId=self.id
        )

        payload = self._convert_snapshot_data_to_dict(snapshot_data)
        snapshot = self.client.initialise_snapshot(payload)
        self.currentSnapshot = snapshot.uuid
        return snapshot

    # get current version of datasource
    def get_current_snapshot(self):
        """
        Description:
            If you want to get current version of a datasource
        :return: SnapshotData class instance of datasource
        """
        snapshot = self.client.get_current_snapshot(self.id)
        if self.currentSnapshot is None:
            self.currentSnapshot = snapshot.uuid
        return snapshot

    def get_asset(self, identifier):
        """"
            Description:
                Find an asset of the datasource
            :param identifier: uid or id of the asset
        """
        return self.client.get_asset(identifier)

    def delete_asset(self, id: int = None):
        if id is None:
            raise TorchSdkException('Id of the asset is required to delete an asset')
        return self.client.delete_asset(id)

    def list_all_snapshots(self):
        return self.client.list_datasource_snapshots(self.id)

    def start_crawler(self):
        return self.client.start_crawler(self.name)

    def get_crawler_status(self):
        return self.client.get_crawler_status(self.name)

    def restart_crawler(self):
        return self.client.restart_crawler(self.name)

    def remove_crawler(self):
        return self.client.remove_crawler(self.name)

    def get_auto_profile_configuration(self):
        return self.client.get_auto_profile_configuration(self.id)

    def remove_auto_profile_configuration(self):
        return self.client.remove_auto_profile_configuration(self.id)


class CrawlerStatus:
    def __init__(self, assemblyName, isSuccess=None, status=None):
        self.assemblyName = assemblyName
        if status is not None:
            self.status = status
        if isSuccess is not None:
            self.isSuccess = isSuccess

    def __repr__(self):
        return f"CrawlerStatus({self.__dict__})"
